from __future__ import annotations
from typing import (MutableSequence, Optional, List, Any)
from ..fable_library.reflection import (TypeInfo, string_type, record_type, int32_type, option_type, array_type, bool_type, obj_type)
from ..fable_library.types import Record

def expr_161() -> TypeInfo:
    return record_type("FsHafas.Raw.RawPltf", [], RawPltf, lambda: [("type", string_type), ("txt", string_type)])


class RawPltf(Record):
    def __init__(self, type: str, txt: str) -> None:
        super().__init__()
        self.type = type
        self.txt = txt
    

RawPltf_reflection = expr_161

def expr_162() -> TypeInfo:
    return record_type("FsHafas.Raw.RawTrnCmpSX", [], RawTrnCmpSX, lambda: [("tc_m", option_type(int32_type)), ("tcoc_x", option_type(array_type(int32_type)))])


class RawTrnCmpSX(Record):
    def __init__(self, tc_m: Optional[int], tcoc_x: Optional[MutableSequence[int]]) -> None:
        super().__init__()
        self.tc_m = tc_m
        self.tcoc_x = tcoc_x
    

RawTrnCmpSX_reflection = expr_162

def expr_163() -> TypeInfo:
    return record_type("FsHafas.Raw.RawDep", [], RawDep, lambda: [("loc_x", option_type(int32_type)), ("idx", option_type(int32_type)), ("d_prod_x", option_type(int32_type)), ("d_platf_s", option_type(string_type)), ("d_in_r", option_type(bool_type)), ("d_time_s", option_type(string_type)), ("d_prog_type", option_type(string_type)), ("d_trn_cmp_sx", option_type(RawTrnCmpSX_reflection())), ("d_tzoffset", option_type(int32_type)), ("type", option_type(string_type)), ("d_time_r", option_type(string_type)), ("d_cncl", option_type(bool_type)), ("d_pltf_s", option_type(RawPltf_reflection())), ("d_platf_r", option_type(string_type)), ("d_pltf_r", option_type(RawPltf_reflection()))])


class RawDep(Record):
    def __init__(self, loc_x: Optional[int], idx: Optional[int], d_prod_x: Optional[int], d_platf_s: Optional[str], d_in_r: Optional[bool], d_time_s: Optional[str], d_prog_type: Optional[str], d_trn_cmp_sx: Optional[RawTrnCmpSX], d_tzoffset: Optional[int], type: Optional[str], d_time_r: Optional[str], d_cncl: Optional[bool], d_pltf_s: Optional[RawPltf], d_platf_r: Optional[str], d_pltf_r: Optional[RawPltf]) -> None:
        super().__init__()
        self.loc_x = loc_x
        self.idx = idx
        self.d_prod_x = d_prod_x
        self.d_platf_s = d_platf_s
        self.d_in_r = d_in_r
        self.d_time_s = d_time_s
        self.d_prog_type = d_prog_type
        self.d_trn_cmp_sx = d_trn_cmp_sx
        self.d_tzoffset = d_tzoffset
        self.type = type
        self.d_time_r = d_time_r
        self.d_cncl = d_cncl
        self.d_pltf_s = d_pltf_s
        self.d_platf_r = d_platf_r
        self.d_pltf_r = d_pltf_r
    

RawDep_reflection = expr_163

def expr_164() -> TypeInfo:
    return record_type("FsHafas.Raw.RawArr", [], RawArr, lambda: [("loc_x", option_type(int32_type)), ("idx", option_type(int32_type)), ("a_platf_s", option_type(string_type)), ("a_out_r", option_type(bool_type)), ("a_time_s", option_type(string_type)), ("a_prog_type", option_type(string_type)), ("a_tzoffset", option_type(int32_type)), ("type", option_type(string_type)), ("a_time_r", option_type(string_type)), ("a_cncl", option_type(bool_type)), ("a_pltf_s", option_type(RawPltf_reflection())), ("a_platf_r", option_type(string_type)), ("a_pltf_r", option_type(RawPltf_reflection())), ("prod_l", option_type(array_type(RawProd_reflection())))])


class RawArr(Record):
    def __init__(self, loc_x: Optional[int], idx: Optional[int], a_platf_s: Optional[str], a_out_r: Optional[bool], a_time_s: Optional[str], a_prog_type: Optional[str], a_tzoffset: Optional[int], type: Optional[str], a_time_r: Optional[str], a_cncl: Optional[bool], a_pltf_s: Optional[RawPltf], a_platf_r: Optional[str], a_pltf_r: Optional[RawPltf], prod_l: Optional[List[RawProd]]) -> None:
        super().__init__()
        self.loc_x = loc_x
        self.idx = idx
        self.a_platf_s = a_platf_s
        self.a_out_r = a_out_r
        self.a_time_s = a_time_s
        self.a_prog_type = a_prog_type
        self.a_tzoffset = a_tzoffset
        self.type = type
        self.a_time_r = a_time_r
        self.a_cncl = a_cncl
        self.a_pltf_s = a_pltf_s
        self.a_platf_r = a_platf_r
        self.a_pltf_r = a_pltf_r
        self.prod_l = prod_l
    

RawArr_reflection = expr_164

def expr_165() -> TypeInfo:
    return record_type("FsHafas.Raw.PubCh", [], PubCh, lambda: [("name", string_type), ("f_date", string_type), ("f_time", string_type), ("t_date", string_type), ("t_time", string_type)])


class PubCh(Record):
    def __init__(self, name: str, f_date: str, f_time: str, t_date: str, t_time: str) -> None:
        super().__init__()
        self.name = name
        self.f_date = f_date
        self.f_time = f_time
        self.t_date = t_date
        self.t_time = t_time
    

PubCh_reflection = expr_165

def expr_166() -> TypeInfo:
    return record_type("FsHafas.Raw.RawHim", [], RawHim, lambda: [("hid", string_type), ("act", bool_type), ("pub", option_type(string_type)), ("head", option_type(string_type)), ("lead", option_type(string_type)), ("text", option_type(string_type)), ("tckr", option_type(string_type)), ("ico_x", int32_type), ("prio", int32_type), ("f_loc_x", option_type(int32_type)), ("t_loc_x", option_type(int32_type)), ("prod", option_type(int32_type)), ("l_mod_date", option_type(string_type)), ("l_mod_time", option_type(string_type)), ("s_date", option_type(string_type)), ("s_time", option_type(string_type)), ("e_date", option_type(string_type)), ("e_time", option_type(string_type)), ("cat", option_type(int32_type)), ("pub_ch_l", array_type(PubCh_reflection())), ("edge_ref_l", option_type(array_type(int32_type))), ("region_ref_l", option_type(array_type(int32_type))), ("cat_ref_l", option_type(array_type(int32_type))), ("event_ref_l", option_type(array_type(int32_type))), ("aff_prod_ref_l", option_type(array_type(int32_type))), ("comp", option_type(string_type))])


class RawHim(Record):
    def __init__(self, hid: str, act: bool, pub: Optional[str], head: Optional[str], lead: Optional[str], text: Optional[str], tckr: Optional[str], ico_x: int, prio: int, f_loc_x: Optional[int], t_loc_x: Optional[int], prod: Optional[int], l_mod_date: Optional[str], l_mod_time: Optional[str], s_date: Optional[str], s_time: Optional[str], e_date: Optional[str], e_time: Optional[str], cat: Optional[int], pub_ch_l: List[PubCh], edge_ref_l: Optional[MutableSequence[int]], region_ref_l: Optional[MutableSequence[int]], cat_ref_l: Optional[MutableSequence[int]], event_ref_l: Optional[MutableSequence[int]], aff_prod_ref_l: Optional[MutableSequence[int]], comp: Optional[str]) -> None:
        super().__init__()
        self.hid = hid
        self.act = act
        self.pub = pub
        self.head = head
        self.lead = lead
        self.text = text
        self.tckr = tckr
        self.ico_x = ico_x or 0
        self.prio = prio or 0
        self.f_loc_x = f_loc_x
        self.t_loc_x = t_loc_x
        self.prod = prod
        self.l_mod_date = l_mod_date
        self.l_mod_time = l_mod_time
        self.s_date = s_date
        self.s_time = s_time
        self.e_date = e_date
        self.e_time = e_time
        self.cat = cat
        self.pub_ch_l = pub_ch_l
        self.edge_ref_l = edge_ref_l
        self.region_ref_l = region_ref_l
        self.cat_ref_l = cat_ref_l
        self.event_ref_l = event_ref_l
        self.aff_prod_ref_l = aff_prod_ref_l
        self.comp = comp
    

RawHim_reflection = expr_166

def expr_167() -> TypeInfo:
    return record_type("FsHafas.Raw.RawMsg", [], RawMsg, lambda: [("type", string_type), ("rem_x", option_type(int32_type)), ("f_loc_x", option_type(int32_type)), ("t_loc_x", option_type(int32_type)), ("f_idx", option_type(int32_type)), ("t_idx", option_type(int32_type)), ("him_x", option_type(int32_type)), ("tag_l", option_type(array_type(string_type)))])


class RawMsg(Record):
    def __init__(self, type: str, rem_x: Optional[int], f_loc_x: Optional[int], t_loc_x: Optional[int], f_idx: Optional[int], t_idx: Optional[int], him_x: Optional[int], tag_l: Optional[List[str]]) -> None:
        super().__init__()
        self.type = type
        self.rem_x = rem_x
        self.f_loc_x = f_loc_x
        self.t_loc_x = t_loc_x
        self.f_idx = f_idx
        self.t_idx = t_idx
        self.him_x = him_x
        self.tag_l = tag_l
    

RawMsg_reflection = expr_167

def expr_168() -> TypeInfo:
    return record_type("FsHafas.Raw.RawRem", [], RawRem, lambda: [("type", string_type), ("code", string_type), ("prio", option_type(int32_type)), ("ico_x", option_type(int32_type)), ("txt_n", option_type(string_type)), ("txt_s", option_type(string_type)), ("jid", option_type(string_type))])


class RawRem(Record):
    def __init__(self, type: str, code: str, prio: Optional[int], ico_x: Optional[int], txt_n: Optional[str], txt_s: Optional[str], jid: Optional[str]) -> None:
        super().__init__()
        self.type = type
        self.code = code
        self.prio = prio
        self.ico_x = ico_x
        self.txt_n = txt_n
        self.txt_s = txt_s
        self.jid = jid
    

RawRem_reflection = expr_168

def expr_169() -> TypeInfo:
    return record_type("FsHafas.Raw.RawStop", [], RawStop, lambda: [("loc_x", int32_type), ("idx", option_type(int32_type)), ("d_prod_x", option_type(int32_type)), ("d_in_r", option_type(bool_type)), ("d_time_s", option_type(string_type)), ("d_time_r", option_type(string_type)), ("d_tzoffset", option_type(int32_type)), ("d_cncl", option_type(bool_type)), ("d_in_s", option_type(bool_type)), ("d_platf_s", option_type(string_type)), ("d_pltf_s", option_type(RawPltf_reflection())), ("d_platf_r", option_type(string_type)), ("d_pltf_r", option_type(RawPltf_reflection())), ("d_prog_type", option_type(string_type)), ("d_dir_txt", option_type(string_type)), ("d_dir_flg", option_type(string_type)), ("d_trn_cmp_sx", option_type(RawTrnCmpSX_reflection())), ("a_prod_x", option_type(int32_type)), ("a_platf_s", option_type(string_type)), ("a_pltf_s", option_type(RawPltf_reflection())), ("a_platf_r", option_type(string_type)), ("a_pltf_r", option_type(RawPltf_reflection())), ("a_out_r", option_type(bool_type)), ("a_time_s", option_type(string_type)), ("a_time_r", option_type(string_type)), ("a_tzoffset", option_type(int32_type)), ("a_cncl", option_type(bool_type)), ("a_out_s", option_type(bool_type)), ("a_platf_ch", option_type(bool_type)), ("a_prog_type", option_type(string_type)), ("type", option_type(string_type)), ("msg_l", option_type(array_type(RawMsg_reflection()))), ("rem_l", option_type(array_type(RawRem_reflection())))])


class RawStop(Record):
    def __init__(self, loc_x: int, idx: Optional[int], d_prod_x: Optional[int], d_in_r: Optional[bool], d_time_s: Optional[str], d_time_r: Optional[str], d_tzoffset: Optional[int], d_cncl: Optional[bool], d_in_s: Optional[bool], d_platf_s: Optional[str], d_pltf_s: Optional[RawPltf], d_platf_r: Optional[str], d_pltf_r: Optional[RawPltf], d_prog_type: Optional[str], d_dir_txt: Optional[str], d_dir_flg: Optional[str], d_trn_cmp_sx: Optional[RawTrnCmpSX], a_prod_x: Optional[int], a_platf_s: Optional[str], a_pltf_s: Optional[RawPltf], a_platf_r: Optional[str], a_pltf_r: Optional[RawPltf], a_out_r: Optional[bool], a_time_s: Optional[str], a_time_r: Optional[str], a_tzoffset: Optional[int], a_cncl: Optional[bool], a_out_s: Optional[bool], a_platf_ch: Optional[bool], a_prog_type: Optional[str], type: Optional[str], msg_l: Optional[List[RawMsg]], rem_l: Optional[List[RawRem]]) -> None:
        super().__init__()
        self.loc_x = loc_x or 0
        self.idx = idx
        self.d_prod_x = d_prod_x
        self.d_in_r = d_in_r
        self.d_time_s = d_time_s
        self.d_time_r = d_time_r
        self.d_tzoffset = d_tzoffset
        self.d_cncl = d_cncl
        self.d_in_s = d_in_s
        self.d_platf_s = d_platf_s
        self.d_pltf_s = d_pltf_s
        self.d_platf_r = d_platf_r
        self.d_pltf_r = d_pltf_r
        self.d_prog_type = d_prog_type
        self.d_dir_txt = d_dir_txt
        self.d_dir_flg = d_dir_flg
        self.d_trn_cmp_sx = d_trn_cmp_sx
        self.a_prod_x = a_prod_x
        self.a_platf_s = a_platf_s
        self.a_pltf_s = a_pltf_s
        self.a_platf_r = a_platf_r
        self.a_pltf_r = a_pltf_r
        self.a_out_r = a_out_r
        self.a_time_s = a_time_s
        self.a_time_r = a_time_r
        self.a_tzoffset = a_tzoffset
        self.a_cncl = a_cncl
        self.a_out_s = a_out_s
        self.a_platf_ch = a_platf_ch
        self.a_prog_type = a_prog_type
        self.type = type
        self.msg_l = msg_l
        self.rem_l = rem_l
    

RawStop_reflection = expr_169

def expr_170() -> TypeInfo:
    return record_type("FsHafas.Raw.PpLocRef", [], PpLocRef, lambda: [("pp_idx", int32_type), ("loc_x", int32_type)])


class PpLocRef(Record):
    def __init__(self, pp_idx: int, loc_x: int) -> None:
        super().__init__()
        self.pp_idx = pp_idx or 0
        self.loc_x = loc_x or 0
    

PpLocRef_reflection = expr_170

def expr_171() -> TypeInfo:
    return record_type("FsHafas.Raw.RawPoly", [], RawPoly, lambda: [("delta", bool_type), ("dim", int32_type), ("type", option_type(string_type)), ("crd_enc_yx", string_type), ("crd_enc_z", option_type(string_type)), ("crd_enc_s", string_type), ("crd_enc_f", string_type), ("pp_loc_ref_l", option_type(array_type(PpLocRef_reflection())))])


class RawPoly(Record):
    def __init__(self, delta: bool, dim: int, type: Optional[str], crd_enc_yx: str, crd_enc_z: Optional[str], crd_enc_s: str, crd_enc_f: str, pp_loc_ref_l: Optional[List[PpLocRef]]) -> None:
        super().__init__()
        self.delta = delta
        self.dim = dim or 0
        self.type = type
        self.crd_enc_yx = crd_enc_yx
        self.crd_enc_z = crd_enc_z
        self.crd_enc_s = crd_enc_s
        self.crd_enc_f = crd_enc_f
        self.pp_loc_ref_l = pp_loc_ref_l
    

RawPoly_reflection = expr_171

def expr_172() -> TypeInfo:
    return record_type("FsHafas.Raw.PolyG", [], PolyG, lambda: [("poly_xl", array_type(int32_type))])


class PolyG(Record):
    def __init__(self, poly_xl: MutableSequence[int]=None) -> None:
        super().__init__()
        self.poly_xl = poly_xl
    

PolyG_reflection = expr_172

def expr_173() -> TypeInfo:
    return record_type("FsHafas.Raw.RawAni", [], RawAni, lambda: [("m_sec", array_type(int32_type)), ("proc", array_type(int32_type)), ("proc_abs", option_type(array_type(int32_type))), ("f_loc_x", array_type(int32_type)), ("t_loc_x", array_type(int32_type)), ("dir_geo", array_type(int32_type)), ("stc_output_x", array_type(int32_type)), ("poly_g", option_type(PolyG_reflection())), ("state", array_type(string_type)), ("poly", option_type(RawPoly_reflection()))])


class RawAni(Record):
    def __init__(self, m_sec: MutableSequence[int], proc: MutableSequence[int], proc_abs: Optional[MutableSequence[int]], f_loc_x: MutableSequence[int], t_loc_x: MutableSequence[int], dir_geo: MutableSequence[int], stc_output_x: MutableSequence[int], poly_g: Optional[PolyG], state: List[str], poly: Optional[RawPoly]) -> None:
        super().__init__()
        self.m_sec = m_sec
        self.proc = proc
        self.proc_abs = proc_abs
        self.f_loc_x = f_loc_x
        self.t_loc_x = t_loc_x
        self.dir_geo = dir_geo
        self.stc_output_x = stc_output_x
        self.poly_g = poly_g
        self.state = state
        self.poly = poly
    

RawAni_reflection = expr_173

def expr_174() -> TypeInfo:
    return record_type("FsHafas.Raw.RawSDays", [], RawSDays, lambda: [("f_loc_x", option_type(int32_type)), ("t_loc_x", option_type(int32_type)), ("s_days_r", option_type(string_type)), ("s_days_i", option_type(string_type)), ("s_days_b", option_type(string_type))])


class RawSDays(Record):
    def __init__(self, f_loc_x: Optional[int], t_loc_x: Optional[int], s_days_r: Optional[str], s_days_i: Optional[str], s_days_b: Optional[str]) -> None:
        super().__init__()
        self.f_loc_x = f_loc_x
        self.t_loc_x = t_loc_x
        self.s_days_r = s_days_r
        self.s_days_i = s_days_i
        self.s_days_b = s_days_b
    

RawSDays_reflection = expr_174

def expr_175() -> TypeInfo:
    return record_type("FsHafas.Raw.RawPolyG", [], RawPolyG, lambda: [("poly_xl", array_type(int32_type))])


class RawPolyG(Record):
    def __init__(self, poly_xl: MutableSequence[int]=None) -> None:
        super().__init__()
        self.poly_xl = poly_xl
    

RawPolyG_reflection = expr_175

def expr_176() -> TypeInfo:
    return record_type("FsHafas.Raw.RawCrd", [], RawCrd, lambda: [("x", int32_type), ("y", int32_type), ("z", option_type(int32_type))])


class RawCrd(Record):
    def __init__(self, x: int, y: int, z: Optional[int]) -> None:
        super().__init__()
        self.x = x or 0
        self.y = y or 0
        self.z = z
    

RawCrd_reflection = expr_176

def expr_177() -> TypeInfo:
    return record_type("FsHafas.Raw.RawFreq", [], RawFreq, lambda: [("jny_l", option_type(array_type(RawJny_reflection()))), ("min_c", option_type(int32_type)), ("max_c", option_type(int32_type)), ("num_c", option_type(int32_type))])


class RawFreq(Record):
    def __init__(self, jny_l: Optional[List[RawJny]], min_c: Optional[int], max_c: Optional[int], num_c: Optional[int]) -> None:
        super().__init__()
        self.jny_l = jny_l
        self.min_c = min_c
        self.max_c = max_c
        self.num_c = num_c
    

RawFreq_reflection = expr_177

def expr_178() -> TypeInfo:
    return record_type("FsHafas.Raw.RawJny", [], RawJny, lambda: [("jid", string_type), ("prod_x", int32_type), ("dir_txt", option_type(string_type)), ("status", option_type(string_type)), ("is_rchbl", option_type(bool_type)), ("ctx_recon", option_type(string_type)), ("rem_l", option_type(array_type(RawRem_reflection()))), ("msg_l", option_type(array_type(RawMsg_reflection()))), ("stb_stop", option_type(RawStop_reflection())), ("subscr", option_type(string_type)), ("poly", option_type(RawPoly_reflection())), ("stop_l", option_type(array_type(RawStop_reflection()))), ("date", option_type(string_type)), ("s_days_l", option_type(array_type(RawSDays_reflection()))), ("d_trn_cmp_sx", option_type(RawTrnCmpSX_reflection())), ("poly_g", option_type(RawPolyG_reflection())), ("ani", option_type(RawAni_reflection())), ("pos", option_type(RawCrd_reflection())), ("freq", option_type(RawFreq_reflection())), ("prod_l", option_type(array_type(RawProd_reflection())))])


class RawJny(Record):
    def __init__(self, jid: str, prod_x: int, dir_txt: Optional[str], status: Optional[str], is_rchbl: Optional[bool], ctx_recon: Optional[str], rem_l: Optional[List[RawRem]], msg_l: Optional[List[RawMsg]], stb_stop: Optional[RawStop], subscr: Optional[str], poly: Optional[RawPoly], stop_l: Optional[List[RawStop]], date: Optional[str], s_days_l: Optional[List[RawSDays]], d_trn_cmp_sx: Optional[RawTrnCmpSX], poly_g: Optional[RawPolyG], ani: Optional[RawAni], pos: Optional[RawCrd], freq: Optional[RawFreq], prod_l: Optional[List[RawProd]]) -> None:
        super().__init__()
        self.jid = jid
        self.prod_x = prod_x or 0
        self.dir_txt = dir_txt
        self.status = status
        self.is_rchbl = is_rchbl
        self.ctx_recon = ctx_recon
        self.rem_l = rem_l
        self.msg_l = msg_l
        self.stb_stop = stb_stop
        self.subscr = subscr
        self.poly = poly
        self.stop_l = stop_l
        self.date = date
        self.s_days_l = s_days_l
        self.d_trn_cmp_sx = d_trn_cmp_sx
        self.poly_g = poly_g
        self.ani = ani
        self.pos = pos
        self.freq = freq
        self.prod_l = prod_l
    

RawJny_reflection = expr_178

def expr_179() -> TypeInfo:
    return record_type("FsHafas.Raw.RawGis", [], RawGis, lambda: [("dist", option_type(int32_type)), ("dur_s", option_type(string_type)), ("dir_geo", option_type(int32_type)), ("ctx", option_type(string_type)), ("gis_prvr", option_type(string_type)), ("get_descr", option_type(bool_type)), ("get_poly", option_type(bool_type)), ("msg_l", option_type(array_type(RawMsg_reflection())))])


class RawGis(Record):
    def __init__(self, dist: Optional[int], dur_s: Optional[str], dir_geo: Optional[int], ctx: Optional[str], gis_prvr: Optional[str], get_descr: Optional[bool], get_poly: Optional[bool], msg_l: Optional[List[RawMsg]]) -> None:
        super().__init__()
        self.dist = dist
        self.dur_s = dur_s
        self.dir_geo = dir_geo
        self.ctx = ctx
        self.gis_prvr = gis_prvr
        self.get_descr = get_descr
        self.get_poly = get_poly
        self.msg_l = msg_l
    

RawGis_reflection = expr_179

def expr_180() -> TypeInfo:
    return record_type("FsHafas.Raw.RawSec", [], RawSec, lambda: [("type", string_type), ("ico_x", option_type(int32_type)), ("dep", RawDep_reflection()), ("arr", RawArr_reflection()), ("jny", option_type(RawJny_reflection())), ("res_state", option_type(string_type)), ("res_recommendation", option_type(string_type)), ("gis", option_type(RawGis_reflection()))])


class RawSec(Record):
    def __init__(self, type: str, ico_x: Optional[int], dep: RawDep, arr: RawArr, jny: Optional[RawJny], res_state: Optional[str], res_recommendation: Optional[str], gis: Optional[RawGis]) -> None:
        super().__init__()
        self.type = type
        self.ico_x = ico_x
        self.dep = dep
        self.arr = arr
        self.jny = jny
        self.res_state = res_state
        self.res_recommendation = res_recommendation
        self.gis = gis
    

RawSec_reflection = expr_180

def expr_181() -> TypeInfo:
    return record_type("FsHafas.Raw.RawSotCtxt", [], RawSotCtxt, lambda: [("cn_loc_x", option_type(int32_type)), ("calc_date", string_type), ("jid", option_type(string_type)), ("loc_mode", string_type), ("p_loc_x", option_type(int32_type)), ("req_mode", string_type), ("sect_x", option_type(int32_type)), ("calc_time", string_type)])


class RawSotCtxt(Record):
    def __init__(self, cn_loc_x: Optional[int], calc_date: str, jid: Optional[str], loc_mode: str, p_loc_x: Optional[int], req_mode: str, sect_x: Optional[int], calc_time: str) -> None:
        super().__init__()
        self.cn_loc_x = cn_loc_x
        self.calc_date = calc_date
        self.jid = jid
        self.loc_mode = loc_mode
        self.p_loc_x = p_loc_x
        self.req_mode = req_mode
        self.sect_x = sect_x
        self.calc_time = calc_time
    

RawSotCtxt_reflection = expr_181

def expr_182() -> TypeInfo:
    return record_type("FsHafas.Raw.Content", [], Content, lambda: [("type", string_type), ("content", string_type)])


class Content(Record):
    def __init__(self, type: str, content: str) -> None:
        super().__init__()
        self.type = type
        self.content = content
    

Content_reflection = expr_182

def expr_183() -> TypeInfo:
    return record_type("FsHafas.Raw.ExtCont", [], ExtCont, lambda: [("content", Content_reflection())])


class ExtCont(Record):
    def __init__(self, content: Content=None) -> None:
        super().__init__()
        self.content = content
    

ExtCont_reflection = expr_183

def expr_184() -> TypeInfo:
    return record_type("FsHafas.Raw.RawTicket", [], RawTicket, lambda: [("name", string_type), ("prc", int32_type), ("cur", string_type), ("ext_cont", ExtCont_reflection())])


class RawTicket(Record):
    def __init__(self, name: str, prc: int, cur: str, ext_cont: ExtCont) -> None:
        super().__init__()
        self.name = name
        self.prc = prc or 0
        self.cur = cur
        self.ext_cont = ext_cont
    

RawTicket_reflection = expr_184

def expr_185() -> TypeInfo:
    return record_type("FsHafas.Raw.RawPrice", [], RawPrice, lambda: [("amount", option_type(int32_type))])


class RawPrice(Record):
    def __init__(self, amount: Optional[int]=None) -> None:
        super().__init__()
        self.amount = amount
    

RawPrice_reflection = expr_185

def expr_186() -> TypeInfo:
    return record_type("FsHafas.Raw.RawFare", [], RawFare, lambda: [("price", option_type(RawPrice_reflection())), ("is_from_price", option_type(bool_type)), ("is_bookable", option_type(bool_type)), ("is_upsell", option_type(bool_type)), ("target_ctx", option_type(string_type)), ("button_text", option_type(string_type)), ("name", option_type(string_type)), ("ticket_l", option_type(array_type(RawTicket_reflection())))])


class RawFare(Record):
    def __init__(self, price: Optional[RawPrice], is_from_price: Optional[bool], is_bookable: Optional[bool], is_upsell: Optional[bool], target_ctx: Optional[str], button_text: Optional[str], name: Optional[str], ticket_l: Optional[List[RawTicket]]) -> None:
        super().__init__()
        self.price = price
        self.is_from_price = is_from_price
        self.is_bookable = is_bookable
        self.is_upsell = is_upsell
        self.target_ctx = target_ctx
        self.button_text = button_text
        self.name = name
        self.ticket_l = ticket_l
    

RawFare_reflection = expr_186

def expr_187() -> TypeInfo:
    return record_type("FsHafas.Raw.RawFareSet", [], RawFareSet, lambda: [("desc", option_type(string_type)), ("fare_l", array_type(RawFare_reflection()))])


class RawFareSet(Record):
    def __init__(self, desc: Optional[str], fare_l: List[RawFare]) -> None:
        super().__init__()
        self.desc = desc
        self.fare_l = fare_l
    

RawFareSet_reflection = expr_187

def expr_188() -> TypeInfo:
    return record_type("FsHafas.Raw.RawTrfRes", [], RawTrfRes, lambda: [("status_code", option_type(string_type)), ("fare_set_l", array_type(RawFareSet_reflection()))])


class RawTrfRes(Record):
    def __init__(self, status_code: Optional[str], fare_set_l: List[RawFareSet]) -> None:
        super().__init__()
        self.status_code = status_code
        self.fare_set_l = fare_set_l
    

RawTrfRes_reflection = expr_188

def expr_189() -> TypeInfo:
    return record_type("FsHafas.Raw.RawOutCon", [], RawOutCon, lambda: [("cid", string_type), ("date", string_type), ("dur", string_type), ("chg", int32_type), ("s_days", RawSDays_reflection()), ("dep", RawDep_reflection()), ("arr", RawArr_reflection()), ("sec_l", array_type(RawSec_reflection())), ("ctx_recon", option_type(string_type)), ("trf_res", option_type(RawTrfRes_reflection())), ("con_subscr", string_type), ("res_state", option_type(string_type)), ("res_recommendation", option_type(string_type)), ("rec_state", string_type), ("sot_rating", option_type(int32_type)), ("is_sot_con", option_type(bool_type)), ("show_arslink", option_type(bool_type)), ("sot_ctxt", option_type(RawSotCtxt_reflection())), ("cksum", string_type), ("msg_l", option_type(array_type(RawMsg_reflection()))), ("freq", option_type(RawFreq_reflection()))])


class RawOutCon(Record):
    def __init__(self, cid: str, date: str, dur: str, chg: int, s_days: RawSDays, dep: RawDep, arr: RawArr, sec_l: List[RawSec], ctx_recon: Optional[str], trf_res: Optional[RawTrfRes], con_subscr: str, res_state: Optional[str], res_recommendation: Optional[str], rec_state: str, sot_rating: Optional[int], is_sot_con: Optional[bool], show_arslink: Optional[bool], sot_ctxt: Optional[RawSotCtxt], cksum: str, msg_l: Optional[List[RawMsg]], freq: Optional[RawFreq]) -> None:
        super().__init__()
        self.cid = cid
        self.date = date
        self.dur = dur
        self.chg = chg or 0
        self.s_days = s_days
        self.dep = dep
        self.arr = arr
        self.sec_l = sec_l
        self.ctx_recon = ctx_recon
        self.trf_res = trf_res
        self.con_subscr = con_subscr
        self.res_state = res_state
        self.res_recommendation = res_recommendation
        self.rec_state = rec_state
        self.sot_rating = sot_rating
        self.is_sot_con = is_sot_con
        self.show_arslink = show_arslink
        self.sot_ctxt = sot_ctxt
        self.cksum = cksum
        self.msg_l = msg_l
        self.freq = freq
    

RawOutCon_reflection = expr_189

def expr_190() -> TypeInfo:
    return record_type("FsHafas.Raw.RawItem", [], RawItem, lambda: [("col", int32_type), ("row", int32_type), ("msg_l", option_type(array_type(RawMsg_reflection()))), ("rem_l", option_type(array_type(int32_type)))])


class RawItem(Record):
    def __init__(self, col: int, row: int, msg_l: Optional[List[RawMsg]], rem_l: Optional[MutableSequence[int]]) -> None:
        super().__init__()
        self.col = col or 0
        self.row = row or 0
        self.msg_l = msg_l
        self.rem_l = rem_l
    

RawItem_reflection = expr_190

def expr_191() -> TypeInfo:
    return record_type("FsHafas.Raw.RawGrid", [], RawGrid, lambda: [("n_cols", int32_type), ("n_rows", int32_type), ("item_l", array_type(RawItem_reflection())), ("type", string_type), ("title", string_type)])


class RawGrid(Record):
    def __init__(self, n_cols: int, n_rows: int, item_l: List[RawItem], type: str, title: str) -> None:
        super().__init__()
        self.n_cols = n_cols or 0
        self.n_rows = n_rows or 0
        self.item_l = item_l
        self.type = type
        self.title = title
    

RawGrid_reflection = expr_191

def expr_192() -> TypeInfo:
    return record_type("FsHafas.Raw.RawLoc", [], RawLoc, lambda: [("lid", option_type(string_type)), ("type", option_type(string_type)), ("name", string_type), ("ico_x", option_type(int32_type)), ("ext_id", option_type(string_type)), ("state", string_type), ("crd", option_type(RawCrd_reflection())), ("p_cls", option_type(int32_type)), ("entry", option_type(bool_type)), ("m_mast_loc_x", option_type(int32_type)), ("p_ref_l", option_type(array_type(int32_type))), ("wt", option_type(int32_type)), ("entry_loc_l", option_type(array_type(int32_type))), ("stop_loc_l", option_type(array_type(int32_type))), ("msg_l", option_type(array_type(RawMsg_reflection()))), ("grid_l", option_type(array_type(RawGrid_reflection()))), ("is_main_mast", option_type(bool_type)), ("meta", option_type(bool_type)), ("dist", option_type(int32_type)), ("dur", option_type(int32_type))])


class RawLoc(Record):
    def __init__(self, lid: Optional[str], type: Optional[str], name: str, ico_x: Optional[int], ext_id: Optional[str], state: str, crd: Optional[RawCrd], p_cls: Optional[int], entry: Optional[bool], m_mast_loc_x: Optional[int], p_ref_l: Optional[MutableSequence[int]], wt: Optional[int], entry_loc_l: Optional[MutableSequence[int]], stop_loc_l: Optional[MutableSequence[int]], msg_l: Optional[List[RawMsg]], grid_l: Optional[List[RawGrid]], is_main_mast: Optional[bool], meta: Optional[bool], dist: Optional[int], dur: Optional[int]) -> None:
        super().__init__()
        self.lid = lid
        self.type = type
        self.name = name
        self.ico_x = ico_x
        self.ext_id = ext_id
        self.state = state
        self.crd = crd
        self.p_cls = p_cls
        self.entry = entry
        self.m_mast_loc_x = m_mast_loc_x
        self.p_ref_l = p_ref_l
        self.wt = wt
        self.entry_loc_l = entry_loc_l
        self.stop_loc_l = stop_loc_l
        self.msg_l = msg_l
        self.grid_l = grid_l
        self.is_main_mast = is_main_mast
        self.meta = meta
        self.dist = dist
        self.dur = dur
    

RawLoc_reflection = expr_192

def expr_193() -> TypeInfo:
    return record_type("FsHafas.Raw.RawProdCtx", [], RawProdCtx, lambda: [("name", option_type(string_type)), ("num", option_type(string_type)), ("match_id", option_type(string_type)), ("cat_out", option_type(string_type)), ("cat_out_s", option_type(string_type)), ("cat_out_l", option_type(string_type)), ("cat_in", option_type(string_type)), ("cat_code", option_type(string_type)), ("admin", option_type(string_type)), ("line_id", option_type(string_type))])


class RawProdCtx(Record):
    def __init__(self, name: Optional[str], num: Optional[str], match_id: Optional[str], cat_out: Optional[str], cat_out_s: Optional[str], cat_out_l: Optional[str], cat_in: Optional[str], cat_code: Optional[str], admin: Optional[str], line_id: Optional[str]) -> None:
        super().__init__()
        self.name = name
        self.num = num
        self.match_id = match_id
        self.cat_out = cat_out
        self.cat_out_s = cat_out_s
        self.cat_out_l = cat_out_l
        self.cat_in = cat_in
        self.cat_code = cat_code
        self.admin = admin
        self.line_id = line_id
    

RawProdCtx_reflection = expr_193

def expr_194() -> TypeInfo:
    return record_type("FsHafas.Raw.RawOp", [], RawOp, lambda: [("name", string_type), ("ico_x", int32_type)])


class RawOp(Record):
    def __init__(self, name: str, ico_x: int) -> None:
        super().__init__()
        self.name = name
        self.ico_x = ico_x or 0
    

RawOp_reflection = expr_194

def expr_195() -> TypeInfo:
    return record_type("FsHafas.Raw.RawProd", [], RawProd, lambda: [("name", option_type(string_type)), ("number", option_type(string_type)), ("ico_x", option_type(int32_type)), ("opr_x", option_type(int32_type)), ("prod_ctx", option_type(RawProdCtx_reflection())), ("cls", option_type(int32_type)), ("line", option_type(string_type)), ("add_name", option_type(string_type)), ("f_loc_x", option_type(int32_type)), ("t_loc_x", option_type(int32_type)), ("prod_x", option_type(int32_type)), ("f_idx", option_type(int32_type)), ("t_idx", option_type(int32_type))])


class RawProd(Record):
    def __init__(self, name: Optional[str], number: Optional[str], ico_x: Optional[int], opr_x: Optional[int], prod_ctx: Optional[RawProdCtx], cls: Optional[int], line: Optional[str], add_name: Optional[str], f_loc_x: Optional[int], t_loc_x: Optional[int], prod_x: Optional[int], f_idx: Optional[int], t_idx: Optional[int]) -> None:
        super().__init__()
        self.name = name
        self.number = number
        self.ico_x = ico_x
        self.opr_x = opr_x
        self.prod_ctx = prod_ctx
        self.cls = cls
        self.line = line
        self.add_name = add_name
        self.f_loc_x = f_loc_x
        self.t_loc_x = t_loc_x
        self.prod_x = prod_x
        self.f_idx = f_idx
        self.t_idx = t_idx
    

RawProd_reflection = expr_195

def expr_196() -> TypeInfo:
    return record_type("FsHafas.Raw.RawRGB", [], RawRGB, lambda: [("r", option_type(int32_type)), ("g", int32_type), ("b", int32_type)])


class RawRGB(Record):
    def __init__(self, r: Optional[int], g: int, b: int) -> None:
        super().__init__()
        self.r = r
        self.g = g or 0
        self.b = b or 0
    

RawRGB_reflection = expr_196

def expr_197() -> TypeInfo:
    return record_type("FsHafas.Raw.RawIco", [], RawIco, lambda: [("res", option_type(string_type)), ("txt", option_type(string_type)), ("text", option_type(string_type)), ("txt_s", option_type(string_type)), ("fg", option_type(RawRGB_reflection())), ("bg", option_type(RawRGB_reflection()))])


class RawIco(Record):
    def __init__(self, res: Optional[str], txt: Optional[str], text: Optional[str], txt_s: Optional[str], fg: Optional[RawRGB], bg: Optional[RawRGB]) -> None:
        super().__init__()
        self.res = res
        self.txt = txt
        self.text = text
        self.txt_s = txt_s
        self.fg = fg
        self.bg = bg
    

RawIco_reflection = expr_197

def expr_198() -> TypeInfo:
    return record_type("FsHafas.Raw.RawDir", [], RawDir, lambda: [("txt", string_type), ("flg", option_type(string_type))])


class RawDir(Record):
    def __init__(self, txt: str, flg: Optional[str]) -> None:
        super().__init__()
        self.txt = txt
        self.flg = flg
    

RawDir_reflection = expr_198

def expr_199() -> TypeInfo:
    return record_type("FsHafas.Raw.RawTcoc", [], RawTcoc, lambda: [("c", string_type), ("r", option_type(int32_type))])


class RawTcoc(Record):
    def __init__(self, c: str, r: Optional[int]) -> None:
        super().__init__()
        self.c = c
        self.r = r
    

RawTcoc_reflection = expr_199

def expr_200() -> TypeInfo:
    return record_type("FsHafas.Raw.RawHimMsgCat", [], RawHimMsgCat, lambda: [("id", int32_type)])


class RawHimMsgCat(Record):
    def __init__(self, id: int=None) -> None:
        super().__init__()
        self.id = id or 0
    

RawHimMsgCat_reflection = expr_200

def expr_201() -> TypeInfo:
    return record_type("FsHafas.Raw.IcoCrd", [], IcoCrd, lambda: [("x", int32_type), ("y", int32_type), ("type", option_type(string_type))])


class IcoCrd(Record):
    def __init__(self, x: int, y: int, type: Optional[str]) -> None:
        super().__init__()
        self.x = x or 0
        self.y = y or 0
        self.type = type
    

IcoCrd_reflection = expr_201

def expr_202() -> TypeInfo:
    return record_type("FsHafas.Raw.RawHimMsgEdge", [], RawHimMsgEdge, lambda: [("f_loc_x", option_type(int32_type)), ("t_loc_x", option_type(int32_type)), ("dir", option_type(int32_type)), ("ico_crd", IcoCrd_reflection()), ("msg_ref_l", option_type(array_type(int32_type))), ("ico_x", option_type(int32_type))])


class RawHimMsgEdge(Record):
    def __init__(self, f_loc_x: Optional[int], t_loc_x: Optional[int], dir: Optional[int], ico_crd: IcoCrd, msg_ref_l: Optional[MutableSequence[int]], ico_x: Optional[int]) -> None:
        super().__init__()
        self.f_loc_x = f_loc_x
        self.t_loc_x = t_loc_x
        self.dir = dir
        self.ico_crd = ico_crd
        self.msg_ref_l = msg_ref_l
        self.ico_x = ico_x
    

RawHimMsgEdge_reflection = expr_202

def expr_203() -> TypeInfo:
    return record_type("FsHafas.Raw.RawHimMsgEvent", [], RawHimMsgEvent, lambda: [("f_loc_x", option_type(int32_type)), ("t_loc_x", option_type(int32_type)), ("f_date", string_type), ("f_time", string_type), ("t_date", string_type), ("t_time", string_type)])


class RawHimMsgEvent(Record):
    def __init__(self, f_loc_x: Optional[int], t_loc_x: Optional[int], f_date: str, f_time: str, t_date: str, t_time: str) -> None:
        super().__init__()
        self.f_loc_x = f_loc_x
        self.t_loc_x = t_loc_x
        self.f_date = f_date
        self.f_time = f_time
        self.t_date = t_date
        self.t_time = t_time
    

RawHimMsgEvent_reflection = expr_203

def expr_204() -> TypeInfo:
    return record_type("FsHafas.Raw.RawCommon", [], RawCommon, lambda: [("loc_l", option_type(array_type(RawLoc_reflection()))), ("prod_l", option_type(array_type(RawProd_reflection()))), ("rem_l", option_type(array_type(RawRem_reflection()))), ("ico_l", option_type(array_type(RawIco_reflection()))), ("op_l", option_type(array_type(RawOp_reflection()))), ("max_c", option_type(int32_type)), ("num_c", option_type(int32_type)), ("him_l", option_type(array_type(RawHim_reflection()))), ("poly_l", option_type(array_type(RawPoly_reflection()))), ("dir_l", option_type(array_type(RawDir_reflection()))), ("tcoc_l", option_type(array_type(RawTcoc_reflection()))), ("him_msg_cat_l", option_type(array_type(RawHimMsgCat_reflection()))), ("him_msg_edge_l", option_type(array_type(RawHimMsgEdge_reflection()))), ("him_msg_event_l", option_type(array_type(RawHimMsgEvent_reflection())))])


class RawCommon(Record):
    def __init__(self, loc_l: Optional[List[RawLoc]], prod_l: Optional[List[RawProd]], rem_l: Optional[List[RawRem]], ico_l: Optional[List[RawIco]], op_l: Optional[List[RawOp]], max_c: Optional[int], num_c: Optional[int], him_l: Optional[List[RawHim]], poly_l: Optional[List[RawPoly]], dir_l: Optional[List[RawDir]], tcoc_l: Optional[List[RawTcoc]], him_msg_cat_l: Optional[List[RawHimMsgCat]], him_msg_edge_l: Optional[List[RawHimMsgEdge]], him_msg_event_l: Optional[List[RawHimMsgEvent]]) -> None:
        super().__init__()
        self.loc_l = loc_l
        self.prod_l = prod_l
        self.rem_l = rem_l
        self.ico_l = ico_l
        self.op_l = op_l
        self.max_c = max_c
        self.num_c = num_c
        self.him_l = him_l
        self.poly_l = poly_l
        self.dir_l = dir_l
        self.tcoc_l = tcoc_l
        self.him_msg_cat_l = him_msg_cat_l
        self.him_msg_edge_l = him_msg_edge_l
        self.him_msg_event_l = him_msg_event_l
    

RawCommon_reflection = expr_204

def expr_205() -> TypeInfo:
    return record_type("FsHafas.Raw.RawMatch", [], RawMatch, lambda: [("field", option_type(string_type)), ("state", option_type(string_type)), ("loc_l", option_type(array_type(RawLoc_reflection())))])


class RawMatch(Record):
    def __init__(self, field: Optional[str], state: Optional[str], loc_l: Optional[List[RawLoc]]) -> None:
        super().__init__()
        self.field = field
        self.state = state
        self.loc_l = loc_l
    

RawMatch_reflection = expr_205

def expr_206() -> TypeInfo:
    return record_type("FsHafas.Raw.RawPos", [], RawPos, lambda: [("loc_x", int32_type), ("dur", int32_type)])


class RawPos(Record):
    def __init__(self, loc_x: int, dur: int) -> None:
        super().__init__()
        self.loc_x = loc_x or 0
        self.dur = dur or 0
    

RawPos_reflection = expr_206

def expr_207() -> TypeInfo:
    return record_type("FsHafas.Raw.RawLine", [], RawLine, lambda: [("line_id", option_type(string_type)), ("prod_x", int32_type), ("dir_ref_l", option_type(array_type(int32_type))), ("jny_l", option_type(array_type(RawJny_reflection())))])


class RawLine(Record):
    def __init__(self, line_id: Optional[str], prod_x: int, dir_ref_l: Optional[MutableSequence[int]], jny_l: Optional[List[RawJny]]) -> None:
        super().__init__()
        self.line_id = line_id
        self.prod_x = prod_x or 0
        self.dir_ref_l = dir_ref_l
        self.jny_l = jny_l
    

RawLine_reflection = expr_207

def expr_208() -> TypeInfo:
    return record_type("FsHafas.Raw.RawResult", [], RawResult, lambda: [("common", option_type(RawCommon_reflection())), ("msg_l", option_type(array_type(RawHim_reflection()))), ("type", option_type(string_type)), ("jny_l", option_type(array_type(RawJny_reflection()))), ("out_con_l", option_type(array_type(RawOutCon_reflection()))), ("out_ctx_scr_b", option_type(string_type)), ("out_ctx_scr_f", option_type(string_type)), ("planrt_ts", option_type(string_type)), ("match", option_type(RawMatch_reflection())), ("loc_l", option_type(array_type(RawLoc_reflection()))), ("journey", option_type(RawJny_reflection())), ("hci_version", option_type(string_type)), ("fp_e", option_type(string_type)), ("s_d", option_type(string_type)), ("s_t", option_type(string_type)), ("fp_b", option_type(string_type)), ("pos_l", option_type(array_type(RawPos_reflection()))), ("line_l", option_type(array_type(RawLine_reflection())))])


class RawResult(Record):
    def __init__(self, common: Optional[RawCommon], msg_l: Optional[List[RawHim]], type: Optional[str], jny_l: Optional[List[RawJny]], out_con_l: Optional[List[RawOutCon]], out_ctx_scr_b: Optional[str], out_ctx_scr_f: Optional[str], planrt_ts: Optional[str], match: Optional[RawMatch], loc_l: Optional[List[RawLoc]], journey: Optional[RawJny], hci_version: Optional[str], fp_e: Optional[str], s_d: Optional[str], s_t: Optional[str], fp_b: Optional[str], pos_l: Optional[List[RawPos]], line_l: Optional[List[RawLine]]) -> None:
        super().__init__()
        self.common = common
        self.msg_l = msg_l
        self.type = type
        self.jny_l = jny_l
        self.out_con_l = out_con_l
        self.out_ctx_scr_b = out_ctx_scr_b
        self.out_ctx_scr_f = out_ctx_scr_f
        self.planrt_ts = planrt_ts
        self.match = match
        self.loc_l = loc_l
        self.journey = journey
        self.hci_version = hci_version
        self.fp_e = fp_e
        self.s_d = s_d
        self.s_t = s_t
        self.fp_b = fp_b
        self.pos_l = pos_l
        self.line_l = line_l
    

RawResult_reflection = expr_208

def expr_209() -> TypeInfo:
    return record_type("FsHafas.Raw.SvcRes", [], SvcRes, lambda: [("meth", string_type), ("err", option_type(string_type)), ("err_txt", option_type(string_type)), ("res", option_type(RawResult_reflection()))])


class SvcRes(Record):
    def __init__(self, meth: str, err: Optional[str], err_txt: Optional[str], res: Optional[RawResult]) -> None:
        super().__init__()
        self.meth = meth
        self.err = err
        self.err_txt = err_txt
        self.res = res
    

SvcRes_reflection = expr_209

def expr_210() -> TypeInfo:
    return record_type("FsHafas.Raw.RawResponse", [], RawResponse, lambda: [("ver", string_type), ("lang", string_type), ("id", option_type(string_type)), ("err", option_type(string_type)), ("err_txt", option_type(string_type)), ("svc_res_l", option_type(array_type(SvcRes_reflection())))])


class RawResponse(Record):
    def __init__(self, ver: str, lang: str, id: Optional[str], err: Optional[str], err_txt: Optional[str], svc_res_l: Optional[List[SvcRes]]) -> None:
        super().__init__()
        self.ver = ver
        self.lang = lang
        self.id = id
        self.err = err
        self.err_txt = err_txt
        self.svc_res_l = svc_res_l
    

RawResponse_reflection = expr_210

def expr_211() -> TypeInfo:
    return record_type("FsHafas.Raw.Cfg", [], Cfg, lambda: [("poly_enc", string_type), ("rt_mode", option_type(string_type))])


class Cfg(Record):
    def __init__(self, poly_enc: str, rt_mode: Optional[str]) -> None:
        super().__init__()
        self.poly_enc = poly_enc
        self.rt_mode = rt_mode
    

Cfg_reflection = expr_211

def expr_212() -> TypeInfo:
    return record_type("FsHafas.Raw.Loc", [], Loc, lambda: [("type", string_type), ("name", option_type(string_type)), ("lid", option_type(string_type))])


class Loc(Record):
    def __init__(self, type: str, name: Optional[str], lid: Optional[str]) -> None:
        super().__init__()
        self.type = type
        self.name = name
        self.lid = lid
    

Loc_reflection = expr_212

def expr_213() -> TypeInfo:
    return record_type("FsHafas.Raw.LocViaInput", [], LocViaInput, lambda: [("loc", Loc_reflection())])


class LocViaInput(Record):
    def __init__(self, loc: Loc=None) -> None:
        super().__init__()
        self.loc = loc
    

LocViaInput_reflection = expr_213

def expr_214() -> TypeInfo:
    return record_type("FsHafas.Raw.LocMatchInput", [], LocMatchInput, lambda: [("loc", Loc_reflection()), ("max_loc", int32_type), ("field", string_type)])


class LocMatchInput(Record):
    def __init__(self, loc: Loc, max_loc: int, field: str) -> None:
        super().__init__()
        self.loc = loc
        self.max_loc = max_loc or 0
        self.field = field
    

LocMatchInput_reflection = expr_214

def expr_215() -> TypeInfo:
    return record_type("FsHafas.Raw.LocMatchRequest", [], LocMatchRequest, lambda: [("input", LocMatchInput_reflection())])


class LocMatchRequest(Record):
    def __init__(self, input: LocMatchInput=None) -> None:
        super().__init__()
        self.input = input
    

LocMatchRequest_reflection = expr_215

def expr_216() -> TypeInfo:
    return record_type("FsHafas.Raw.LineMatchRequest", [], LineMatchRequest, lambda: [("input", string_type)])


class LineMatchRequest(Record):
    def __init__(self, input: str=None) -> None:
        super().__init__()
        self.input = input
    

LineMatchRequest_reflection = expr_216

def expr_217() -> TypeInfo:
    return record_type("FsHafas.Raw.JourneyDetailsRequest", [], JourneyDetailsRequest, lambda: [("jid", string_type), ("name", string_type), ("get_polyline", bool_type)])


class JourneyDetailsRequest(Record):
    def __init__(self, jid: str, name: str, get_polyline: bool) -> None:
        super().__init__()
        self.jid = jid
        self.name = name
        self.get_polyline = get_polyline
    

JourneyDetailsRequest_reflection = expr_217

def expr_218() -> TypeInfo:
    return record_type("FsHafas.Raw.JnyFltr", [], JnyFltr, lambda: [("type", string_type), ("mode", string_type), ("value", option_type(string_type)), ("meta", option_type(string_type))])


class JnyFltr(Record):
    def __init__(self, type: str, mode: str, value: Optional[str], meta: Optional[str]) -> None:
        super().__init__()
        self.type = type
        self.mode = mode
        self.value = value
        self.meta = meta
    

JnyFltr_reflection = expr_218

def expr_219() -> TypeInfo:
    return record_type("FsHafas.Raw.TvlrProf", [], TvlrProf, lambda: [("type", string_type), ("redtn_card", option_type(int32_type))])


class TvlrProf(Record):
    def __init__(self, type: str, redtn_card: Optional[int]) -> None:
        super().__init__()
        self.type = type
        self.redtn_card = redtn_card
    

TvlrProf_reflection = expr_219

def expr_220() -> TypeInfo:
    return record_type("FsHafas.Raw.TrfReq", [], TrfReq, lambda: [("jny_cl", int32_type), ("tvlr_prof", array_type(TvlrProf_reflection())), ("c_type", string_type)])


class TrfReq(Record):
    def __init__(self, jny_cl: int, tvlr_prof: List[TvlrProf], c_type: str) -> None:
        super().__init__()
        self.jny_cl = jny_cl or 0
        self.tvlr_prof = tvlr_prof
        self.c_type = c_type
    

TrfReq_reflection = expr_220

def expr_221() -> TypeInfo:
    return record_type("FsHafas.Raw.StationBoardRequest", [], StationBoardRequest, lambda: [("type", string_type), ("date", string_type), ("time", string_type), ("stb_loc", Loc_reflection()), ("jny_fltr_l", array_type(JnyFltr_reflection())), ("dur", int32_type)])


class StationBoardRequest(Record):
    def __init__(self, type: str, date: str, time: str, stb_loc: Loc, jny_fltr_l: List[JnyFltr], dur: int) -> None:
        super().__init__()
        self.type = type
        self.date = date
        self.time = time
        self.stb_loc = stb_loc
        self.jny_fltr_l = jny_fltr_l
        self.dur = dur or 0
    

StationBoardRequest_reflection = expr_221

def expr_222() -> TypeInfo:
    return record_type("FsHafas.Raw.HimSearchRequest", [], HimSearchRequest, lambda: [("him_fltr_l", array_type(JnyFltr_reflection())), ("get_polyline", bool_type), ("max_num", int32_type), ("date_b", string_type), ("time_b", string_type)])


class HimSearchRequest(Record):
    def __init__(self, him_fltr_l: List[JnyFltr], get_polyline: bool, max_num: int, date_b: str, time_b: str) -> None:
        super().__init__()
        self.him_fltr_l = him_fltr_l
        self.get_polyline = get_polyline
        self.max_num = max_num or 0
        self.date_b = date_b
        self.time_b = time_b
    

HimSearchRequest_reflection = expr_222

def expr_223() -> TypeInfo:
    return record_type("FsHafas.Raw.ReconstructionRequest", [], ReconstructionRequest, lambda: [("get_ist", bool_type), ("get_passlist", bool_type), ("get_polyline", bool_type), ("get_tariff", bool_type), ("ctx_recon", option_type(string_type))])


class ReconstructionRequest(Record):
    def __init__(self, get_ist: bool, get_passlist: bool, get_polyline: bool, get_tariff: bool, ctx_recon: Optional[str]) -> None:
        super().__init__()
        self.get_ist = get_ist
        self.get_passlist = get_passlist
        self.get_polyline = get_polyline
        self.get_tariff = get_tariff
        self.ctx_recon = ctx_recon
    

ReconstructionRequest_reflection = expr_223

def expr_224() -> TypeInfo:
    return record_type("FsHafas.Raw.LocData", [], LocData, lambda: [("loc", Loc_reflection()), ("type", string_type), ("date", string_type), ("time", string_type)])


class LocData(Record):
    def __init__(self, loc: Loc, type: str, date: str, time: str) -> None:
        super().__init__()
        self.loc = loc
        self.type = type
        self.date = date
        self.time = time
    

LocData_reflection = expr_224

def expr_225() -> TypeInfo:
    return record_type("FsHafas.Raw.SearchOnTripRequest", [], SearchOnTripRequest, lambda: [("sot_mode", string_type), ("jid", string_type), ("loc_data", LocData_reflection()), ("arr_loc_l", array_type(Loc_reflection())), ("jny_fltr_l", array_type(JnyFltr_reflection())), ("get_passlist", bool_type), ("get_polyline", bool_type), ("min_chg_time", int32_type), ("get_tariff", bool_type)])


class SearchOnTripRequest(Record):
    def __init__(self, sot_mode: str, jid: str, loc_data: LocData, arr_loc_l: List[Loc], jny_fltr_l: List[JnyFltr], get_passlist: bool, get_polyline: bool, min_chg_time: int, get_tariff: bool) -> None:
        super().__init__()
        self.sot_mode = sot_mode
        self.jid = jid
        self.loc_data = loc_data
        self.arr_loc_l = arr_loc_l
        self.jny_fltr_l = jny_fltr_l
        self.get_passlist = get_passlist
        self.get_polyline = get_polyline
        self.min_chg_time = min_chg_time or 0
        self.get_tariff = get_tariff
    

SearchOnTripRequest_reflection = expr_225

def expr_226() -> TypeInfo:
    return record_type("FsHafas.Raw.TripSearchRequest", [], TripSearchRequest, lambda: [("get_passlist", bool_type), ("max_chg", int32_type), ("min_chg_time", int32_type), ("dep_loc_l", array_type(Loc_reflection())), ("via_loc_l", option_type(array_type(LocViaInput_reflection()))), ("arr_loc_l", array_type(Loc_reflection())), ("jny_fltr_l", array_type(JnyFltr_reflection())), ("gis_fltr_l", array_type(JnyFltr_reflection())), ("get_tariff", bool_type), ("ushrp", bool_type), ("get_pt", bool_type), ("get_iv", bool_type), ("get_polyline", bool_type), ("out_date", string_type), ("out_time", string_type), ("num_f", int32_type), ("out_frwd", bool_type), ("trf_req", option_type(TrfReq_reflection()))])


class TripSearchRequest(Record):
    def __init__(self, get_passlist: bool, max_chg: int, min_chg_time: int, dep_loc_l: List[Loc], via_loc_l: Optional[List[LocViaInput]], arr_loc_l: List[Loc], jny_fltr_l: List[JnyFltr], gis_fltr_l: List[JnyFltr], get_tariff: bool, ushrp: bool, get_pt: bool, get_iv: bool, get_polyline: bool, out_date: str, out_time: str, num_f: int, out_frwd: bool, trf_req: Optional[TrfReq]) -> None:
        super().__init__()
        self.get_passlist = get_passlist
        self.max_chg = max_chg or 0
        self.min_chg_time = min_chg_time or 0
        self.dep_loc_l = dep_loc_l
        self.via_loc_l = via_loc_l
        self.arr_loc_l = arr_loc_l
        self.jny_fltr_l = jny_fltr_l
        self.gis_fltr_l = gis_fltr_l
        self.get_tariff = get_tariff
        self.ushrp = ushrp
        self.get_pt = get_pt
        self.get_iv = get_iv
        self.get_polyline = get_polyline
        self.out_date = out_date
        self.out_time = out_time
        self.num_f = num_f or 0
        self.out_frwd = out_frwd
        self.trf_req = trf_req
    

TripSearchRequest_reflection = expr_226

def expr_227() -> TypeInfo:
    return record_type("FsHafas.Raw.JourneyMatchRequest", [], JourneyMatchRequest, lambda: [("input", string_type), ("date", option_type(string_type))])


class JourneyMatchRequest(Record):
    def __init__(self, input: str, date: Optional[str]) -> None:
        super().__init__()
        self.input = input
        self.date = date
    

JourneyMatchRequest_reflection = expr_227

def expr_228() -> TypeInfo:
    return record_type("FsHafas.Raw.RawcCrd", [], RawcCrd, lambda: [("x", int32_type), ("y", int32_type)])


class RawcCrd(Record):
    def __init__(self, x: int, y: int) -> None:
        super().__init__()
        self.x = x or 0
        self.y = y or 0
    

RawcCrd_reflection = expr_228

def expr_229() -> TypeInfo:
    return record_type("FsHafas.Raw.RawRing", [], RawRing, lambda: [("c_crd", RawcCrd_reflection()), ("max_dist", int32_type), ("min_dist", int32_type)])


class RawRing(Record):
    def __init__(self, c_crd: RawcCrd, max_dist: int, min_dist: int) -> None:
        super().__init__()
        self.c_crd = c_crd
        self.max_dist = max_dist or 0
        self.min_dist = min_dist or 0
    

RawRing_reflection = expr_229

def expr_230() -> TypeInfo:
    return record_type("FsHafas.Raw.LocGeoPosRequest", [], LocGeoPosRequest, lambda: [("ring", RawRing_reflection()), ("loc_fltr_l", array_type(JnyFltr_reflection())), ("get_pois", bool_type), ("get_stops", bool_type), ("max_loc", int32_type)])


class LocGeoPosRequest(Record):
    def __init__(self, ring: RawRing, loc_fltr_l: List[JnyFltr], get_pois: bool, get_stops: bool, max_loc: int) -> None:
        super().__init__()
        self.ring = ring
        self.loc_fltr_l = loc_fltr_l
        self.get_pois = get_pois
        self.get_stops = get_stops
        self.max_loc = max_loc or 0
    

LocGeoPosRequest_reflection = expr_230

def expr_231() -> TypeInfo:
    return record_type("FsHafas.Raw.LocGeoReachRequest", [], LocGeoReachRequest, lambda: [("loc", Loc_reflection()), ("max_dur", int32_type), ("max_chg", int32_type), ("date", string_type), ("time", string_type), ("period", int32_type), ("jny_fltr_l", array_type(JnyFltr_reflection()))])


class LocGeoReachRequest(Record):
    def __init__(self, loc: Loc, max_dur: int, max_chg: int, date: str, time: str, period: int, jny_fltr_l: List[JnyFltr]) -> None:
        super().__init__()
        self.loc = loc
        self.max_dur = max_dur or 0
        self.max_chg = max_chg or 0
        self.date = date
        self.time = time
        self.period = period or 0
        self.jny_fltr_l = jny_fltr_l
    

LocGeoReachRequest_reflection = expr_231

def expr_232() -> TypeInfo:
    return record_type("FsHafas.Raw.LocDetailsRequest", [], LocDetailsRequest, lambda: [("loc_l", array_type(Loc_reflection()))])


class LocDetailsRequest(Record):
    def __init__(self, loc_l: List[Loc]=None) -> None:
        super().__init__()
        self.loc_l = loc_l
    

LocDetailsRequest_reflection = expr_232

def expr_233() -> TypeInfo:
    return record_type("FsHafas.Raw.ServerInfoRequest", [], ServerInfoRequest, lambda: [("get_version_info", bool_type)])


class ServerInfoRequest(Record):
    def __init__(self, get_version_info: bool=None) -> None:
        super().__init__()
        self.get_version_info = get_version_info
    

ServerInfoRequest_reflection = expr_233

def expr_234() -> TypeInfo:
    return record_type("FsHafas.Raw.RawRect", [], RawRect, lambda: [("ll_crd", RawCrd_reflection()), ("ur_crd", RawCrd_reflection())])


class RawRect(Record):
    def __init__(self, ll_crd: RawCrd, ur_crd: RawCrd) -> None:
        super().__init__()
        self.ll_crd = ll_crd
        self.ur_crd = ur_crd
    

RawRect_reflection = expr_234

def expr_235() -> TypeInfo:
    return record_type("FsHafas.Raw.JourneyGeoPosRequest", [], JourneyGeoPosRequest, lambda: [("max_jny", int32_type), ("only_rt", bool_type), ("date", string_type), ("time", string_type), ("rect", RawRect_reflection()), ("per_size", int32_type), ("per_step", int32_type), ("age_of_report", bool_type), ("jny_fltr_l", array_type(JnyFltr_reflection())), ("train_pos_mode", string_type)])


class JourneyGeoPosRequest(Record):
    def __init__(self, max_jny: int, only_rt: bool, date: str, time: str, rect: RawRect, per_size: int, per_step: int, age_of_report: bool, jny_fltr_l: List[JnyFltr], train_pos_mode: str) -> None:
        super().__init__()
        self.max_jny = max_jny or 0
        self.only_rt = only_rt
        self.date = date
        self.time = time
        self.rect = rect
        self.per_size = per_size or 0
        self.per_step = per_step or 0
        self.age_of_report = age_of_report
        self.jny_fltr_l = jny_fltr_l
        self.train_pos_mode = train_pos_mode
    

JourneyGeoPosRequest_reflection = expr_235

def expr_236() -> TypeInfo:
    return record_type("FsHafas.Raw.SvcReq", [], SvcReq, lambda: [("cfg", Cfg_reflection()), ("meth", string_type), ("req", obj_type)])


class SvcReq(Record):
    def __init__(self, cfg: Cfg, meth: str, req: Any) -> None:
        super().__init__()
        self.cfg = cfg
        self.meth = meth
        self.req = req
    

SvcReq_reflection = expr_236

def expr_237() -> TypeInfo:
    return record_type("FsHafas.Raw.RawRequestClient", [], RawRequestClient, lambda: [("id", string_type), ("v", string_type), ("type", string_type), ("name", string_type)])


class RawRequestClient(Record):
    def __init__(self, id: str, v: str, type: str, name: str) -> None:
        super().__init__()
        self.id = id
        self.v = v
        self.type = type
        self.name = name
    

RawRequestClient_reflection = expr_237

def expr_238() -> TypeInfo:
    return record_type("FsHafas.Raw.RawRequestAuth", [], RawRequestAuth, lambda: [("type", string_type), ("aid", string_type)])


class RawRequestAuth(Record):
    def __init__(self, type: str, aid: str) -> None:
        super().__init__()
        self.type = type
        self.aid = aid
    

RawRequestAuth_reflection = expr_238

def expr_239() -> TypeInfo:
    return record_type("FsHafas.Raw.RawRequest", [], RawRequest, lambda: [("lang", string_type), ("svc_req_l", array_type(SvcReq_reflection())), ("client", RawRequestClient_reflection()), ("ext", string_type), ("ver", string_type), ("auth", RawRequestAuth_reflection())])


class RawRequest(Record):
    def __init__(self, lang: str, svc_req_l: List[SvcReq], client: RawRequestClient, ext: str, ver: str, auth: RawRequestAuth) -> None:
        super().__init__()
        self.lang = lang
        self.svc_req_l = svc_req_l
        self.client = client
        self.ext = ext
        self.ver = ver
        self.auth = auth
    

RawRequest_reflection = expr_239




import re
from datetime import datetime, timedelta, timezone
from typing import Match, Optional

from .types import FSharpRef
from .util import DateKind

formatRegExp = re.compile(r"(\w)\1*")


def op_subtraction(x: datetime, y: datetime) -> timedelta:
    return x - y


def create(
    year: int,
    month: int,
    day: int,
    h: int = 0,
    m: int = 0,
    s: int = 0,
    ms: int = 0,
    kind: Optional[DateKind] = None,
) -> datetime:
    if kind == DateKind.UTC:
        date = datetime(
            year=year,
            month=month,
            day=day,
            hour=h,
            minute=m,
            second=s,
            microsecond=ms * 1000,
            tzinfo=timezone.utc,
        )
    else:
        date = datetime(year, month, day, h, m, s, ms * 1000)

    return date


def year(d: datetime) -> int:
    return d.year


def date_to_string_with_custom_format(date: datetime, format: str, utc: bool) -> str:
    def match(m: Match[str]) -> str:
        match = m.group()
        m = match[:1]
        # print(match)

        rep = None
        if m == "y":
            y = date.astimezone(timezone.utc).year if utc else date.year
            rep = y % 100 if len(match) < 4 else y
        elif m == "M":
            rep = date.astimezone(timezone.utc).month if utc else date.month
        elif m == "H":
            rep = date.astimezone(timezone.utc).hour if utc else date.hour
        elif m == "m":
            rep = date.astimezone(timezone.utc).minute if utc else date.minute
        elif m == "s":
            rep = date.astimezone(timezone.utc).second if utc else date.second
        elif m == "f":
            rep = date.astimezone(timezone.utc).microsecond if utc else date.microsecond
            rep = rep // 1000

        if rep:
            return f"0{rep}" if (rep < 10 and len(match) > 1) else f"{rep}"
        else:
            return match
        return ""

    ret = formatRegExp.sub(match, format)
    return ret

    # return format.replace(/(\w)\1*/g, (match) => {
    #     let rep = Number.NaN;
    #     switch (match.substring(0, 1)) {
    #         case "y":
    #             const y = utc ? date.getUTCFullYear() : date.getFullYear();
    #             rep = match.length < 4 ? y % 100 : y;
    #             break;
    #         case "M":
    #             rep = (utc ? date.getUTCMonth() : date.getMonth()) + 1;
    #             break;
    #         case "d":
    #             rep = utc ? date.getUTCDate() : date.getDate();
    #             break;
    #         case "H":
    #             rep = utc ? date.getUTCHours() : date.getHours();
    #             break;
    #         case "h":
    #             const h = utc ? date.getUTCHours() : date.getHours();
    #             rep = h > 12 ? h % 12 : h;
    #             break;
    #         case "m":
    #             rep = utc ? date.getUTCMinutes() : date.getMinutes();
    #             break;
    #         case "s":
    #             rep = utc ? date.getUTCSeconds() : date.getSeconds();
    #             break;
    #         case "f":
    #             rep = utc ? date.getUTCMilliseconds() : date.getMilliseconds();
    #             break;
    #     }
    #     if (Number.isNaN(rep)) {
    #         return match;
    #     }
    #     else {
    #         return (rep < 10 && match.length > 1) ? "0" + rep : "" + rep;
    #     }


# def dateToStringWithOffset(date, format=None):
#     d = new Date(date.getTime() + ((_a = date.offset) !== null && _a !== void 0 ? _a : 0));
#     if (typeof format !== "string") {
#         return d.toISOString().replace(/\.\d+/, "").replace(/[A-Z]|\.\d+/g, " ") + dateOffsetToString(((_b = date.offset) !== null && _b !== void 0 ? _b : 0));
#     }
#     else if (format.length === 1) {
#         switch (format) {
#             case "D":
#             case "d": return dateToHalfUTCString(d, "first");
#             case "T":
#             case "t": return dateToHalfUTCString(d, "second");
#             case "O":
#             case "o": return dateToISOStringWithOffset(d, ((_c = date.offset) !== null && _c !== void 0 ? _c : 0));
#             default: throw new Error("Unrecognized Date print format");
#         }

#     else:
#         return dateToStringWithCustomFormat(d, format, True)


def date_to_string_with_kind(date: datetime, format: Optional[str] = None) -> str:
    utc = date.tzinfo == timezone.utc
    if not format:
        return date.isoformat() if utc else str(date)

    elif len(format) == 1:
        if format == "D" or format == "d":
            return dateToHalfUTCString(date, "first") if utc else str(date.date())
        elif format == "T" or format == "t":
            return dateToHalfUTCString(date, "second") if utc else str(date.time())
        elif format == "O" or format == "o":
            return dateToISOString(date, utc)
        else:
            raise Exception("Unrecognized Date print format")

    else:
        return date_to_string_with_custom_format(date, format, utc)


def to_string(date: datetime, format: Optional[str] = None, provider=None) -> str:
    if date.tzinfo:
        return date_to_string_with_offset(date, format)

    return date_to_string_with_kind(date, format)


def now() -> datetime:
    return datetime.now()


def utc_now() -> datetime:
    return datetime.utcnow()


def to_local_time(date: datetime) -> datetime:
    return date.astimezone()


def compare(x: datetime, y: datetime) -> int:
    if x == y:
        return 0

    if x < y:
        return -1

    return 1


def equals(x: datetime, y: datetime) -> bool:
    return x == y


def max_value() -> datetime:
    return datetime.max


def min_value() -> datetime:
    return datetime.min


def op_addition(x: datetime, y: timedelta) -> datetime:
    return x + y


def parse(string: str, detectUTC: bool = False) -> datetime:
    from dateutil import parser

    return parser.parse(string)


def try_parse(
    string: str, style: int, unsigned: bool, bitsize: int, defValue: FSharpRef[datetime]
) -> bool:
    try:
        defValue.contents = parse(string)
        return True
    except Exception:
        return False



from __future__ import annotations
from array import array as array_3
import polyline
import math
from typing import (MutableSequence, List, Optional, Any, Tuple)
from ...fable_library.array import (try_find, map, sum, map_indexed as map_indexed_1)
from ...fable_library.option import value
from ...fable_library.seq import (to_array, map_indexed)
from ...fable_library.types import Float64Array
from ...fable_library.util import round as round_1
from ...fs_hafas_python.context import Context
from ...fs_hafas_python.types_hafas_client import (FeatureCollection, Stop, Geometry, Feature)
from ...fs_hafas_python.types_raw_hafas_client import (RawPoly, PpLocRef)
from .common import get_element_at

def decode(s: str) -> List[MutableSequence[float]]:
    return []


def round(f: float) -> float:
    return round_1(f, 5)


default_feature_collection : FeatureCollection = FeatureCollection("FeatureCollection", [])

def get_stop(ctx: Context, p: RawPoly, i: int) -> Any:
    match_value : Optional[List[PpLocRef]] = p.pp_loc_ref_l
    if match_value is None:
        pass
    
    else: 
        def predicate(p_loc_ref_l: PpLocRef, ctx: Context=ctx, p: RawPoly=p, i: int=i) -> bool:
            return p_loc_ref_l.pp_idx == i
        
        match_value_1 : Optional[PpLocRef] = try_find(predicate, match_value)
        if match_value_1 is None:
            pass
        
        else: 
            match_value_2 : Optional[Any] = get_element_at(match_value_1.loc_x, ctx.common.locations)
            (pattern_matching_result, s) = (None, None)
            if match_value_2 is not None:
                if isinstance(value(match_value_2), Stop):
                    pattern_matching_result = 0
                    s = value(match_value_2)
                
                else: 
                    pattern_matching_result = 1
                
            
            else: 
                pattern_matching_result = 1
            
            if pattern_matching_result == 0:
                return s
            
        
    


def parse_polyline(ctx: Context, poly: RawPoly) -> FeatureCollection:
    def mapping(i: int, p: MutableSequence[float], ctx: Context=ctx, poly: RawPoly=poly) -> Feature:
        return Feature("Feature", get_stop(ctx, poly, i), Geometry("Point", array_3("d", [round(p[1]), round(p[0])])))
    
    return FeatureCollection(default_feature_collection.type, to_array(map_indexed(mapping, polyline.decode(poly.crd_enc_yx))))


def calculate_distance(p1latitude: float, p1longitude: float, p2latitude: float, p2longitude: float) -> float:
    d_lat : float = ((p2latitude - p1latitude) * 3.141592653589793) / 180
    d_lon : float = ((p2longitude - p1longitude) * 3.141592653589793) / 180
    lat1 : float = (p1latitude * 3.141592653589793) / 180
    lat2 : float = (p2latitude * 3.141592653589793) / 180
    a : float = (math.sin(d_lat / 2) * math.sin(d_lat / 2)) + (((math.sin(d_lon / 2) * math.sin(d_lon / 2)) * math.cos(lat1)) * math.cos(lat2))
    return 6371 * (2 * math.atan2(math.sqrt(a), math.sqrt(1 - a)))


def distance_of_feature_collection(fc: FeatureCollection) -> float:
    def mapping(f: Feature, fc: FeatureCollection=fc) -> Tuple[float, float]:
        return (f.geometry.coordinates[1], f.geometry.coordinates[0])
    
    lat_lon_points : List[Tuple[float, float]] = map(mapping, fc.features, None)
    def mapping_1(i: int, _arg1: Tuple[float, float], fc: FeatureCollection=fc) -> float:
        if i > 0:
            prev : Tuple[float, float] = lat_lon_points[i - 1]
            curr : Tuple[float, float] = lat_lon_points[i]
            return calculate_distance(prev[0], prev[1], curr[0], curr[1])
        
        else: 
            return 0
        
    
    class ObjectExpr437:
        @property
        def GetZero(self) -> Any:
            def arrow_435(__unit: Any=None) -> int:
                return 0
            
            return arrow_435
        
        @property
        def Add(self) -> Any:
            def arrow_436(x: float, y: float) -> float:
                return x + y
            
            return arrow_436
        
    return sum(map_indexed_1(mapping_1, lat_lon_points, Float64Array), ObjectExpr437())





from typing import (TypeVar, Any, Optional, Callable, List, Tuple)
from .list import FSharpList
from .map_util import (add_to_set, try_get_value, get_item_from_dict, add_to_dict)
from .mutable_map import Dictionary
from .mutable_set import HashSet
from .seq import (delay, filter, map, to_array, to_list)
from .types import FSharpRef
from .util import (IEnumerable, IEqualityComparer, get_enumerator)

_T = TypeVar("_T")

_KEY = TypeVar("_KEY")

def distinct(xs: IEnumerable[_T], comparer: IEqualityComparer[Any]) -> IEnumerable[_T]:
    def arrow_126(xs: IEnumerable[_T]=xs, comparer: IEqualityComparer[Any]=comparer) -> IEnumerable[_T]:
        hash_set : Any = HashSet([], comparer)
        def predicate(x: Optional[_T]=None) -> bool:
            return add_to_set(x, hash_set)
        
        return filter(predicate, xs)
    
    return delay(arrow_126)


def distinct_by(projection: Callable[[_T], _KEY], xs: IEnumerable[_T], comparer: IEqualityComparer[Any]) -> IEnumerable[_T]:
    def arrow_127(projection: Callable[[_T], _KEY]=projection, xs: IEnumerable[_T]=xs, comparer: IEqualityComparer[Any]=comparer) -> IEnumerable[_T]:
        hash_set : Any = HashSet([], comparer)
        def predicate(x: Optional[_T]=None) -> bool:
            return add_to_set(projection(x), hash_set)
        
        return filter(predicate, xs)
    
    return delay(arrow_127)


def except_(items_to_exclude: IEnumerable[_T], xs: IEnumerable[_T], comparer: IEqualityComparer[Any]) -> IEnumerable[_T]:
    def arrow_128(items_to_exclude: IEnumerable[_T]=items_to_exclude, xs: IEnumerable[_T]=xs, comparer: IEqualityComparer[Any]=comparer) -> IEnumerable[_T]:
        hash_set : Any = HashSet(items_to_exclude, comparer)
        def predicate(x: Optional[_T]=None) -> bool:
            return add_to_set(x, hash_set)
        
        return filter(predicate, xs)
    
    return delay(arrow_128)


def count_by(projection: Callable[[_T], _KEY], xs: IEnumerable[_T], comparer: IEqualityComparer[Any]) -> IEnumerable[Tuple[_KEY, int]]:
    def arrow_132(projection: Callable[[_T], _KEY]=projection, xs: IEnumerable[_T]=xs, comparer: IEqualityComparer[Any]=comparer) -> IEnumerable[Tuple[_KEY, int]]:
        dict_1 : Any = Dictionary([], comparer)
        keys : List[_KEY] = []
        with get_enumerator(xs) as enumerator:
            while enumerator.System_Collections_IEnumerator_MoveNext():
                key : _KEY = projection(enumerator.System_Collections_Generic_IEnumerator_00601_get_Current())
                match_value : Tuple[bool, int]
                out_arg : int = 0
                def arrow_129(__unit: Any=None) -> int:
                    return out_arg
                
                def arrow_130(v: int) -> None:
                    nonlocal out_arg
                    out_arg = v or 0
                
                match_value = (try_get_value(dict_1, key, FSharpRef(arrow_129, arrow_130)), out_arg)
                if match_value[0]:
                    dict_1[key] = match_value[1] + 1
                
                else: 
                    dict_1[key] = 1
                    (keys.append(key))
                
        def arrow_131(key_1: Optional[_KEY]=None) -> Tuple[_KEY, int]:
            return (key_1, get_item_from_dict(dict_1, key_1))
        
        return map(arrow_131, keys)
    
    return delay(arrow_132)


def group_by(projection: Callable[[_T], _KEY], xs: IEnumerable[_T], comparer: IEqualityComparer[Any]) -> IEnumerable[Tuple[_KEY, IEnumerable[_T]]]:
    def arrow_136(projection: Callable[[_T], _KEY]=projection, xs: IEnumerable[_T]=xs, comparer: IEqualityComparer[Any]=comparer) -> IEnumerable[Tuple[_KEY, IEnumerable[_T]]]:
        dict_1 : Any = Dictionary([], comparer)
        keys : List[_KEY] = []
        with get_enumerator(xs) as enumerator:
            while enumerator.System_Collections_IEnumerator_MoveNext():
                x : _T = enumerator.System_Collections_Generic_IEnumerator_00601_get_Current()
                key : _KEY = projection(x)
                match_value : Tuple[bool, List[_T]]
                out_arg : List[_T] = None
                def arrow_133(__unit: Any=None) -> List[_T]:
                    return out_arg
                
                def arrow_134(v: List[_T]) -> None:
                    nonlocal out_arg
                    out_arg = v
                
                match_value = (try_get_value(dict_1, key, FSharpRef(arrow_133, arrow_134)), out_arg)
                if match_value[0]:
                    (match_value[1].append(x))
                
                else: 
                    add_to_dict(dict_1, key, [x])
                    (keys.append(key))
                
        def arrow_135(key_1: Optional[_KEY]=None) -> Tuple[_KEY, IEnumerable[_T]]:
            return (key_1, get_item_from_dict(dict_1, key_1))
        
        return map(arrow_135, keys)
    
    return delay(arrow_136)


def Array_distinct(xs: List[_T], comparer: IEqualityComparer[Any]) -> List[_T]:
    return to_array(distinct(xs, comparer))


def Array_distinctBy(projection: Callable[[_T], _KEY], xs: List[_T], comparer: IEqualityComparer[Any]) -> List[_T]:
    return to_array(distinct_by(projection, xs, comparer))


def Array_except(items_to_exclude: IEnumerable[_T], xs: List[_T], comparer: IEqualityComparer[Any]) -> List[_T]:
    return to_array(except_(items_to_exclude, xs, comparer))


def Array_countBy(projection: Callable[[_T], _KEY], xs: List[_T], comparer: IEqualityComparer[Any]) -> List[Tuple[_KEY, int]]:
    return to_array(count_by(projection, xs, comparer))


def Array_groupBy(projection: Callable[[_T], _KEY], xs: List[_T], comparer: IEqualityComparer[Any]) -> List[Tuple[_KEY, List[_T]]]:
    def mapping(tupled_arg: Tuple[_KEY, IEnumerable[_T]], projection: Callable[[_T], _KEY]=projection, xs: List[_T]=xs, comparer: IEqualityComparer[Any]=comparer) -> Tuple[_KEY, List[_T]]:
        return (tupled_arg[0], to_array(tupled_arg[1]))
    
    return to_array(map(mapping, group_by(projection, xs, comparer)))


def List_distinct(xs: FSharpList[_T], comparer: IEqualityComparer[Any]) -> FSharpList[_T]:
    return to_list(distinct(xs, comparer))


def List_distinctBy(projection: Callable[[_T], _KEY], xs: FSharpList[_T], comparer: IEqualityComparer[Any]) -> FSharpList[_T]:
    return to_list(distinct_by(projection, xs, comparer))


def List_except(items_to_exclude: IEnumerable[_T], xs: FSharpList[_T], comparer: IEqualityComparer[Any]) -> FSharpList[_T]:
    return to_list(except_(items_to_exclude, xs, comparer))


def List_countBy(projection: Callable[[_T], _KEY], xs: FSharpList[_T], comparer: IEqualityComparer[Any]) -> FSharpList[Tuple[_KEY, int]]:
    return to_list(count_by(projection, xs, comparer))


def List_groupBy(projection: Callable[[_T], _KEY], xs: FSharpList[_T], comparer: IEqualityComparer[Any]) -> FSharpList[Tuple[_KEY, FSharpList[_T]]]:
    def mapping(tupled_arg: Tuple[_KEY, IEnumerable[_T]], projection: Callable[[_T], _KEY]=projection, xs: FSharpList[_T]=xs, comparer: IEqualityComparer[Any]=comparer) -> Tuple[_KEY, FSharpList[_T]]:
        return (tupled_arg[0], to_list(tupled_arg[1]))
    
    return to_list(map(mapping, group_by(projection, xs, comparer)))





from abc import abstractmethod
from typing import (Any, Protocol)

class IExports(Protocol):
    @abstractmethod
    def chdir(self, __arg0: str) -> None:
        ...
    
    @abstractmethod
    def chroot(self, __arg0: str) -> None:
        ...
    
    @abstractmethod
    def close(self, fd: int) -> None:
        ...
    
    @property
    @abstractmethod
    def environ(self) -> Any:
        ...
    
    @abstractmethod
    def getcwd(self) -> str:
        ...
    
    @abstractmethod
    def kill(self, pid: int, sig: int) -> None:
        ...
    
    @abstractmethod
    def putenv(self, key: str, value: str) -> None:
        ...
    




from __future__ import annotations

from abc import abstractmethod
from collections import deque
from threading import Lock, RLock, Timer
from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    Iterable,
    Optional,
    Protocol,
    TypeVar,
    overload,
)

from .util import IDisposable

_T = TypeVar("_T")
_U = TypeVar("_U")
_D = TypeVar("_D", bound=IDisposable)


class OperationCanceledError(Exception):
    def __init__(self, msg: Optional[str] = None) -> None:
        super().__init__(msg or "The operation was canceled")


class _Listener(Protocol):
    @overload
    def __call__(self) -> None:
        ...

    @overload
    def __call__(self, state: Any) -> None:
        ...

    def __call__(self, __state: Optional[Any] = None) -> None:
        ...


class CancellationToken:
    def __init__(self, cancelled: bool = False):
        self.cancelled = cancelled
        self.listeners: Dict[int, Callable[[], None]] = {}
        self.idx = 0
        self.lock = RLock()

    @property
    def is_cancelled(self):
        return self.cancelled

    is_cancellation_requested = is_cancelled

    def cancel(self) -> None:
        cancel = False
        with self.lock:
            if not self.cancelled:
                cancel = True
                self.cancelled = True

        if cancel:
            for listener in self.listeners.values():
                listener()

    def add_listener(self, f: Callable[[], None]) -> int:
        with self.lock:
            id = self.idx
            self.idx = self.idx + 1
            self.listeners[id] = f

        return id

    def remove_listener(self, id: int) -> None:
        with self.lock:
            del self.listeners[id]

    def register(self, f: _Listener, state: Optional[Any] = None) -> None:
        if state:
            id = self.add_listener(lambda: f(state))
        else:
            id = self.add_listener(f)

        def dispose():
            self.remove_listener(id)

        IDisposable.create(dispose)


class IAsyncContext(Generic[_T]):
    @abstractmethod
    def on_success(self, value: Optional[_T] = None) -> None:
        ...

    @abstractmethod
    def on_error(self, error: Exception) -> None:
        ...

    @abstractmethod
    def on_cancel(self, error: OperationCanceledError) -> None:
        ...

    @property
    @abstractmethod
    def trampoline(self) -> "Trampoline":
        ...

    @trampoline.setter
    @abstractmethod
    def trampoline(self, val: Trampoline):
        ...

    @property
    @abstractmethod
    def cancel_token(self) -> CancellationToken:
        ...

    @cancel_token.setter
    @abstractmethod
    def cancel_token(self, val: CancellationToken):
        ...

    @staticmethod
    def create(
        on_success: Optional[Callable[[Optional[_T]], None]],
        on_error: Optional[Callable[[Exception], None]],
        on_cancel: Optional[Callable[[OperationCanceledError], None]],
        trampoline: Optional[Trampoline],
        cancel_token: Optional[CancellationToken],
    ) -> AnonymousAsyncContext[_T]:
        return AnonymousAsyncContext(
            on_success, on_error, on_cancel, trampoline, cancel_token
        )


""" FSharpAsync"""
Async = Callable[[IAsyncContext[_T]], None]


def empty_continuation(x: Any = None) -> None:
    pass


class AnonymousAsyncContext(IAsyncContext[_T]):
    def __init__(
        self,
        on_success: Optional[Callable[[Optional[_T]], None]] = None,
        on_error: Optional[Callable[[Exception], None]] = None,
        on_cancel: Optional[Callable[[OperationCanceledError], None]] = None,
        trampoline: Optional[Trampoline] = None,
        cancel_token: Optional[CancellationToken] = None,
    ):
        self._on_success: Callable[[Optional[_T]], None] = (
            on_success or empty_continuation
        )
        self._on_error: Callable[[Exception], None] = on_error or empty_continuation
        self._on_cancel: Callable[[OperationCanceledError], None] = (
            on_cancel or empty_continuation
        )

        self._cancel_token = cancel_token
        self._trampoline = trampoline

    def on_success(self, value: Optional[_T] = None) -> None:
        return self._on_success(value)

    def on_error(self, error: Exception) -> None:
        return self._on_error(error)

    def on_cancel(self, error: OperationCanceledError) -> None:
        return self._on_cancel(error)

    @property
    @abstractmethod
    def trampoline(self) -> Trampoline:
        return self._trampoline

    @trampoline.setter
    @abstractmethod
    def trampoline(self, val: Trampoline):
        self._trampoline = val

    @property
    @abstractmethod
    def cancel_token(self) -> CancellationToken:
        return self._cancel_token

    @cancel_token.setter
    @abstractmethod
    def cancel_token(self, val: CancellationToken):
        self._cancel_token = val


class Trampoline:
    MaxTrampolineCallCount = 150  # Max recursion depth: 1000

    def __init__(self):
        self.call_count: int = 0
        self.lock = Lock()
        self.queue: deque[Callable[[], None]] = deque()
        self.running: bool = False

    def increment_and_check(self):
        with self.lock:
            self.call_count = self.call_count + 1
            return self.call_count > Trampoline.MaxTrampolineCallCount

    def run(self, action: Callable[[], None]):

        if self.increment_and_check():
            with self.lock:
                self.queue.append(action)

            if not self.running:
                self.running = True
                timer = Timer(0.0, self._run)
                timer.start()
        else:
            action()

    def _run(self) -> None:
        while len(self.queue):
            with self.lock:
                self.call_count = 0
                action = self.queue.popleft()

            action()

        self.running = False


def protected_cont(f: Async[_T]) -> Async[_T]:
    def _protected_cont(ctx: IAsyncContext[_T]):
        if ctx.cancel_token and ctx.cancel_token.is_cancelled:
            ctx.on_cancel(OperationCanceledError())

        def fn():
            try:
                return f(ctx)
            except Exception as err:
                # print("Exception: ", err)
                ctx.on_error(err)

        ctx.trampoline.run(fn)

    return _protected_cont


def protected_bind(
    computation: Callable[[IAsyncContext[_T]], None], binder: Callable[[_T], Async[_U]]
) -> Async[_U]:
    def cont(ctx: IAsyncContext[_T]):
        def on_success(x: Optional[_T]) -> None:
            try:
                binder(x)(ctx)
            except Exception as err:
                # print("Exception: ", err)
                ctx.on_error(err)

        ctx_ = IAsyncContext.create(
            on_success, ctx.on_error, ctx.on_cancel, ctx.trampoline, ctx.cancel_token
        )
        return computation(ctx_)

    return protected_cont(cont)


def protected_return(value: Optional[_T] = None) -> Async[_T]:
    f: Callable[[IAsyncContext[_T]], None] = lambda ctx: ctx.on_success(value)
    return protected_cont(f)


class AsyncBuilder:
    def Bind(
        self, computation: Async[_T], binder: Callable[[_T], Async[_U]]
    ) -> Async[_U]:
        return protected_bind(computation, binder)

    def Combine(self, computation1: Async[Any], computation2: Async[_T]) -> Async[_T]:
        binder: Callable[[Optional[_T]], Async[_T]] = lambda _: computation2
        return self.Bind(computation1, binder)

    def Delay(self, generator: Callable[[], Async[_T]]) -> Async[_T]:
        return protected_cont(lambda ctx: generator()(ctx))

    def For(self, sequence: Iterable[_T], body: Callable[[_T], Async[_U]]) -> Async[_U]:
        done: bool = False
        it = iter(sequence)
        try:
            cur = next(it)
        except StopIteration:
            done = True

        def delay():
            nonlocal cur, done
            res = body(cur)
            try:
                cur = next(it)
            except StopIteration:
                done = True
            return res

        return self.While(lambda: not done, self.Delay(delay))

    def Return(self, value: Optional[_T] = None) -> Async[_T]:
        return protected_return(value)

    def ReturnFrom(self, computation: Async[_T]) -> Async[_T]:
        return computation

    def TryFinally(
        self, computation: Async[_T], compensation: Callable[[], None]
    ) -> Async[_T]:
        def cont(ctx: IAsyncContext[_T]) -> None:
            def on_success(x: Optional[_T] = None) -> None:
                compensation()
                ctx.on_success(x)

            def on_error(x: Exception) -> None:
                compensation()
                ctx.on_error(x)

            def on_cancel(x: OperationCanceledError) -> None:
                compensation()
                ctx.on_cancel(x)

            ctx_ = IAsyncContext.create(
                on_success, on_error, on_cancel, ctx.trampoline, ctx.cancel_token
            )
            computation(ctx_)

        return protected_cont(cont)

    def TryWith(
        self, computation: Async[_T], catch_handler: Callable[[Exception], Async[_T]]
    ) -> Async[_T]:
        def fn(ctx: IAsyncContext[_T]):
            def on_error(err: Exception) -> None:
                try:
                    catch_handler(err)(ctx)
                except Exception as ex2:
                    ctx.on_error(ex2)

            ctx_ = IAsyncContext.create(
                on_success=ctx.on_success,
                on_cancel=ctx.on_cancel,
                cancel_token=ctx.cancel_token,
                trampoline=ctx.trampoline,
                on_error=on_error,
            )

            return computation(ctx_)

        return protected_cont(fn)

    def Using(self, resource: _D, binder: Callable[[_D], Async[_U]]) -> Async[_U]:
        compensation: Callable[[], None] = lambda: resource.Dispose()
        return self.TryFinally(binder(resource), compensation)

    def While(self, guard: Callable[[], bool], computation: Async[_T]) -> Async[_T]:
        if guard():
            binder: Callable[[Optional[_T]], Async[_T]] = lambda _: self.While(
                guard, computation
            )
            return self.Bind(computation, binder)
        else:
            return self.Return()

    def Zero(self) -> Async[Any]:
        return protected_cont(lambda ctx: ctx.on_success())


singleton = AsyncBuilder()



from __future__ import annotations
import builtins
import functools
import math
from typing import (TypeVar, List, Callable, Optional, Tuple, Any, MutableSequence)
from .global_ import (SR_indexOutOfBounds, IGenericAdder_1, IGenericAverager_1)
from .native import (Cons_1, Helpers_allocateArrayFromCons, Helpers_fillImpl, Helpers_indexOfImpl, Helpers_spliceImpl)
from .option import (some, default_arg, value as value_1)
from .util import (max as max_1, compare_primitives, IEnumerable, IEqualityComparer, ignore, get_enumerator, equals as equals_1, IComparer, min as min_1)

_A_ = TypeVar("_A_")

_T = TypeVar("_T")

_U = TypeVar("_U")

_T1 = TypeVar("_T1")

_T2 = TypeVar("_T2")

_T3 = TypeVar("_T3")

_STATE = TypeVar("_STATE")

_RESULT = TypeVar("_RESULT")

_B_ = TypeVar("_B_")

_A = TypeVar("_A")

_B = TypeVar("_B")

_C_ = TypeVar("_C_")

def index_not_found() -> _A_:
    raise Exception("An index satisfying the predicate was not found in the collection.")


def different_lengths() -> _A_:
    raise Exception("Arrays had different lengths")


def append(array1: List[_T], array2: List[_T], cons: Cons_1[_T]) -> List[_T]:
    len1 : int = len(array1) or 0
    len2 : int = len(array2) or 0
    new_array : List[_T] = Helpers_allocateArrayFromCons(cons, len1 + len2)
    for i in range(0, (len1 - 1) + 1, 1):
        new_array[i] = array1[i]
    for i_1 in range(0, (len2 - 1) + 1, 1):
        new_array[i_1 + len1] = array2[i_1]
    return new_array


def filter(predicate: Callable[[_T], bool], array: List[_T]) -> List[_T]:
    return list(builtins.filter(predicate, array))


def fill(target: List[_T], target_index: int, count: int, value: _T) -> List[_T]:
    return Helpers_fillImpl(target, value, target_index, count)


def get_sub_array(array: List[_T], start: int, count: int) -> List[_T]:
    return array[start:start+count]


def last(array: List[_T]) -> _T:
    if len(array) == 0:
        raise Exception("The input array was empty\\nParameter name: array")
    
    return array[len(array) - 1]


def try_last(array: List[_T]) -> Optional[_T]:
    if len(array) == 0:
        return None
    
    else: 
        return some(array[len(array) - 1])
    


def map_indexed(f: Callable[[int, _T], _U], source: List[_T], cons: Cons_1[_U]) -> List[_U]:
    len_1 : int = len(source) or 0
    target : List[_U] = Helpers_allocateArrayFromCons(cons, len_1)
    for i in range(0, (len_1 - 1) + 1, 1):
        target[i] = f(i, source[i])
    return target


def map(f: Callable[[_T], _U], source: List[_T], cons: Cons_1[_U]) -> List[_U]:
    len_1 : int = len(source) or 0
    target : List[_U] = Helpers_allocateArrayFromCons(cons, len_1)
    for i in range(0, (len_1 - 1) + 1, 1):
        target[i] = f(source[i])
    return target


def map_indexed2(f: Callable[[int, _T1, _T2], _U], source1: List[_T1], source2: List[_T2], cons: Cons_1[_U]) -> List[_U]:
    if len(source1) != len(source2):
        raise Exception("Arrays had different lengths")
    
    result : List[_U] = Helpers_allocateArrayFromCons(cons, len(source1))
    for i in range(0, (len(source1) - 1) + 1, 1):
        result[i] = f(i, source1[i], source2[i])
    return result


def map2(f: Callable[[_T1, _T2], _U], source1: List[_T1], source2: List[_T2], cons: Cons_1[_U]) -> List[_U]:
    if len(source1) != len(source2):
        raise Exception("Arrays had different lengths")
    
    result : List[_U] = Helpers_allocateArrayFromCons(cons, len(source1))
    for i in range(0, (len(source1) - 1) + 1, 1):
        result[i] = f(source1[i], source2[i])
    return result


def map_indexed3(f: Callable[[int, _T1, _T2, _T3], _U], source1: List[_T1], source2: List[_T2], source3: List[_T3], cons: Cons_1[_U]) -> List[_U]:
    if True if (len(source1) != len(source2)) else (len(source2) != len(source3)):
        raise Exception("Arrays had different lengths")
    
    result : List[_U] = Helpers_allocateArrayFromCons(cons, len(source1))
    for i in range(0, (len(source1) - 1) + 1, 1):
        result[i] = f(i, source1[i], source2[i], source3[i])
    return result


def map3(f: Callable[[_T1, _T2, _T3], _U], source1: List[_T1], source2: List[_T2], source3: List[_T3], cons: Cons_1[_U]) -> List[_U]:
    if True if (len(source1) != len(source2)) else (len(source2) != len(source3)):
        raise Exception("Arrays had different lengths")
    
    result : List[_U] = Helpers_allocateArrayFromCons(cons, len(source1))
    for i in range(0, (len(source1) - 1) + 1, 1):
        result[i] = f(source1[i], source2[i], source3[i])
    return result


def map_fold(mapping: Callable[[_STATE, _T], Tuple[_RESULT, _STATE]], state: _STATE, array: List[_T], cons: Cons_1[_RESULT]) -> Tuple[List[_RESULT], _STATE]:
    match_value : int = len(array) or 0
    if match_value == 0:
        return ([], state)
    
    else: 
        acc : _STATE = state
        res : List[_RESULT] = Helpers_allocateArrayFromCons(cons, match_value)
        for i in range(0, (len(array) - 1) + 1, 1):
            pattern_input : Tuple[_RESULT, _STATE] = mapping(acc, array[i])
            res[i] = pattern_input[0]
            acc = pattern_input[1]
        return (res, acc)
    


def map_fold_back(mapping: Callable[[_T, _STATE], Tuple[_RESULT, _STATE]], array: List[_T], state: _STATE, cons: Cons_1[_RESULT]) -> Tuple[List[_RESULT], _STATE]:
    match_value : int = len(array) or 0
    if match_value == 0:
        return ([], state)
    
    else: 
        acc : _STATE = state
        res : List[_RESULT] = Helpers_allocateArrayFromCons(cons, match_value)
        for i in range(len(array) - 1, 0 - 1, -1):
            pattern_input : Tuple[_RESULT, _STATE] = mapping(array[i], acc)
            res[i] = pattern_input[0]
            acc = pattern_input[1]
        return (res, acc)
    


def indexed(source: List[_T]) -> List[Tuple[int, _T]]:
    len_1 : int = len(source) or 0
    target : List[Tuple[int, _T]] = [None]*len_1
    for i in range(0, (len_1 - 1) + 1, 1):
        target[i] = (i, source[i])
    return target


def truncate(count: int, array: List[_T]) -> List[_T]:
    def arrow_47(x: int, y: int, count: int=count, array: List[_T]=array) -> int:
        return compare_primitives(x, y)
    
    count_1 : int = max_1(arrow_47, 0, count) or 0
    return array[0:0+count_1]


def concat(arrays: IEnumerable[List[_T]], cons: Cons_1[_T]) -> List[_T]:
    arrays_1 : List[List[_T]] = arrays if (isinstance(arrays, list)) else (list(arrays))
    match_value : int = len(arrays_1) or 0
    if match_value == 0:
        return Helpers_allocateArrayFromCons(cons, 0)
    
    elif match_value == 1:
        return arrays_1[0]
    
    else: 
        total_idx : int = 0
        total_length : int = 0
        for idx in range(0, (len(arrays_1) - 1) + 1, 1):
            arr_1 : List[_T] = arrays_1[idx]
            total_length = (total_length + len(arr_1)) or 0
        result : List[_T] = Helpers_allocateArrayFromCons(cons, total_length)
        for idx_1 in range(0, (len(arrays_1) - 1) + 1, 1):
            arr_2 : List[_T] = arrays_1[idx_1]
            for j in range(0, (len(arr_2) - 1) + 1, 1):
                result[total_idx] = arr_2[j]
                total_idx = (total_idx + 1) or 0
        return result
    


def collect(mapping: Callable[[_T], List[_U]], array: List[_T], cons: Cons_1[_U]) -> List[_U]:
    return concat(map(mapping, array, None), cons)


def where(predicate: Callable[[_A_], bool], array: List[_A_]) -> List[_A_]:
    return list(builtins.filter(predicate, array))


def contains(value: _T, array: List[_T], eq: IEqualityComparer[Any]) -> bool:
    def loop(i_mut: int, value: _T=value, array: List[_T]=array, eq: IEqualityComparer[Any]=eq) -> bool:
        while True:
            (i,) = (i_mut,)
            if i >= len(array):
                return False
            
            elif eq.Equals(value, array[i]):
                return True
            
            else: 
                i_mut = i + 1
                continue
            
            break
    
    return loop(0)


def empty(cons: Cons_1[_A_]) -> List[_A_]:
    return Helpers_allocateArrayFromCons(cons, 0)


def singleton(value: _T, cons: Cons_1[_T]) -> List[_T]:
    ar : List[_T] = Helpers_allocateArrayFromCons(cons, 1)
    ar[0] = value
    return ar


def initialize(count: int, initializer: Callable[[int], _T], cons: Cons_1[_T]) -> List[_T]:
    if count < 0:
        raise Exception("The input must be non-negative\\nParameter name: count")
    
    result : List[_T] = Helpers_allocateArrayFromCons(cons, count)
    for i in range(0, (count - 1) + 1, 1):
        result[i] = initializer(i)
    return result


def pairwise(array: List[_T]) -> List[Tuple[_T, _T]]:
    if len(array) < 2:
        return []
    
    else: 
        count : int = (len(array) - 1) or 0
        result : List[Tuple[_T, _T]] = [None]*count
        for i in range(0, (count - 1) + 1, 1):
            result[i] = (array[i], array[i + 1])
        return result
    


def replicate(count: int, initial: _T, cons: Cons_1[_T]) -> List[_T]:
    if count < 0:
        raise Exception("The input must be non-negative\\nParameter name: count")
    
    result : List[_T] = Helpers_allocateArrayFromCons(cons, count)
    for i in range(0, (len(result) - 1) + 1, 1):
        result[i] = initial
    return result


def copy(array: List[_T]) -> List[_T]:
    return array[:]


def reverse(array: List[_T]) -> List[_T]:
    array_1 : List[_T] = array[:]
    return array_1[::-1]


def scan(folder: Callable[[_STATE, _T], _STATE], state: _STATE, array: List[_T], cons: Cons_1[_STATE]) -> List[_STATE]:
    res : List[_STATE] = Helpers_allocateArrayFromCons(cons, len(array) + 1)
    res[0] = state
    for i in range(0, (len(array) - 1) + 1, 1):
        res[i + 1] = folder(res[i], array[i])
    return res


def scan_back(folder: Callable[[_T, _STATE], _STATE], array: List[_T], state: _STATE, cons: Cons_1[_STATE]) -> List[_STATE]:
    res : List[_STATE] = Helpers_allocateArrayFromCons(cons, len(array) + 1)
    res[len(array)] = state
    for i in range(len(array) - 1, 0 - 1, -1):
        res[i] = folder(array[i], res[i + 1])
    return res


def skip(count: int, array: List[_T], cons: Cons_1[_T]) -> List[_T]:
    if count > len(array):
        raise Exception("count is greater than array length\\nParameter name: count")
    
    if count == len(array):
        return Helpers_allocateArrayFromCons(cons, 0)
    
    else: 
        count_1 : int = (0 if (count < 0) else count) or 0
        return array[count_1:]
    


def skip_while(predicate: Callable[[_T], bool], array: List[_T], cons: Cons_1[_T]) -> List[_T]:
    count : int = 0
    while predicate(array[count]) if (count < len(array)) else False:
        count = (count + 1) or 0
    if count == len(array):
        return Helpers_allocateArrayFromCons(cons, 0)
    
    else: 
        return array[count:]
    


def take(count: int, array: List[_T], cons: Cons_1[_T]) -> List[_T]:
    if count < 0:
        raise Exception("The input must be non-negative\\nParameter name: count")
    
    if count > len(array):
        raise Exception("count is greater than array length\\nParameter name: count")
    
    if count == 0:
        return Helpers_allocateArrayFromCons(cons, 0)
    
    else: 
        return array[0:0+count]
    


def take_while(predicate: Callable[[_T], bool], array: List[_T], cons: Cons_1[_T]) -> List[_T]:
    count : int = 0
    while predicate(array[count]) if (count < len(array)) else False:
        count = (count + 1) or 0
    if count == 0:
        return Helpers_allocateArrayFromCons(cons, 0)
    
    else: 
        return array[0:0+count]
    


def add_in_place(x: _T, array: List[_T]) -> None:
    ignore(array.append(x))


def add_range_in_place(range: IEnumerable[_T], array: List[_T]) -> None:
    with get_enumerator(range) as enumerator:
        while enumerator.System_Collections_IEnumerator_MoveNext():
            add_in_place(enumerator.System_Collections_Generic_IEnumerator_00601_get_Current(), array)


def insert_range_in_place(index: int, range: IEnumerable[_T], array: List[_T]) -> None:
    i : int = index or 0
    with get_enumerator(range) as enumerator:
        while enumerator.System_Collections_IEnumerator_MoveNext():
            x : _T = enumerator.System_Collections_Generic_IEnumerator_00601_get_Current()
            def arrow_48(index: int=index, range: IEnumerable[_T]=range, array: List[_T]=array) -> List[_T]:
                index_1 : int = i or 0
                return array.insert(index_1, x)
            
            ignore(arrow_48())
            i = (i + 1) or 0


def remove_in_place(item_1: _T, array: List[_T]) -> bool:
    i : int = Helpers_indexOfImpl(array, item_1, 0) or 0
    if i > -1:
        ignore(Helpers_spliceImpl(array, i, 1))
        return True
    
    else: 
        return False
    


def remove_all_in_place(predicate: Callable[[_T], bool], array: List[_T]) -> int:
    def count_remove_all(count: int, predicate: Callable[[_T], bool]=predicate, array: List[_T]=array) -> int:
        i : int = (next((i for i, x in enumerate(array) if (predicate)(x)), -1)) or 0
        if i > -1:
            ignore(Helpers_spliceImpl(array, i, 1))
            return count_remove_all(count) + 1
        
        else: 
            return count
        
    
    return count_remove_all(0)


def copy_to(source: List[_T], source_index: int, target: List[_T], target_index: int, count: int) -> None:
    diff : int = (target_index - source_index) or 0
    for i in range(source_index, ((source_index + count) - 1) + 1, 1):
        target[i + diff] = source[i]


def copy_to_typed_array(source: List[_T], source_index: int, target: List[_T], target_index: int, count: int) -> None:
    try: 
        target.set(source.subarray(source_index, source_index + count), target_index)
    
    except Exception as match_value:
        copy_to(source, source_index, target, target_index, count)
    


def index_of(array: List[_T], item_1: _T, start: Optional[int]=None, count: Optional[int]=None) -> int:
    start_1 : int = default_arg(start, 0) or 0
    i : int = Helpers_indexOfImpl(array, item_1, start_1) or 0
    if (i >= (start_1 + value_1(count))) if (count is not None) else False:
        return -1
    
    else: 
        return i
    


def partition(f: Callable[[_T], bool], source: List[_T], cons: Cons_1[_T]) -> Tuple[List[_T], List[_T]]:
    len_1 : int = len(source) or 0
    res1 : List[_T] = Helpers_allocateArrayFromCons(cons, len_1)
    res2 : List[_T] = Helpers_allocateArrayFromCons(cons, len_1)
    i_true : int = 0
    i_false : int = 0
    for i in range(0, (len_1 - 1) + 1, 1):
        if f(source[i]):
            res1[i_true] = source[i]
            i_true = (i_true + 1) or 0
        
        else: 
            res2[i_false] = source[i]
            i_false = (i_false + 1) or 0
        
    return (truncate(i_true, res1), truncate(i_false, res2))


def find(predicate: Callable[[_T], bool], array: List[_T]) -> _T:
    match_value : Optional[_T] = next((x for x in array if (predicate)(x)), None)
    if match_value is None:
        return index_not_found()
    
    else: 
        return value_1(match_value)
    


def try_find(predicate: Callable[[_T], bool], array: List[_T]) -> Optional[_T]:
    return next((x for x in array if (predicate)(x)), None)


def find_index(predicate: Callable[[_T], bool], array: List[_T]) -> int:
    match_value : int = (next((i for i, x in enumerate(array) if (predicate)(x)), -1)) or 0
    if match_value > -1:
        return match_value
    
    else: 
        return index_not_found()
    


def try_find_index(predicate: Callable[[_T], bool], array: List[_T]) -> Optional[int]:
    match_value : int = (next((i for i, x in enumerate(array) if (predicate)(x)), -1)) or 0
    if match_value > -1:
        return match_value
    
    else: 
        return None
    


def pick(chooser: Callable[[_A_], Optional[_B_]], array: List[_A_]) -> _B_:
    def loop(i_mut: int, chooser: Callable[[_A_], Optional[_B_]]=chooser, array: List[_A_]=array) -> _B_:
        while True:
            (i,) = (i_mut,)
            if i >= len(array):
                return index_not_found()
            
            else: 
                match_value : Optional[_B_] = chooser(array[i])
                if match_value is not None:
                    return value_1(match_value)
                
                else: 
                    i_mut = i + 1
                    continue
                
            
            break
    
    return loop(0)


def try_pick(chooser: Callable[[_A_], Optional[_B_]], array: List[_A_]) -> Optional[_B_]:
    def loop(i_mut: int, chooser: Callable[[_A_], Optional[_B_]]=chooser, array: List[_A_]=array) -> Optional[_B_]:
        while True:
            (i,) = (i_mut,)
            if i >= len(array):
                return None
            
            else: 
                match_value : Optional[_B_] = chooser(array[i])
                if match_value is None:
                    i_mut = i + 1
                    continue
                
                else: 
                    return match_value
                
            
            break
    
    return loop(0)


def find_back(predicate: Callable[[_A_], bool], array: List[_A_]) -> _A_:
    def loop(i_mut: int, predicate: Callable[[_A_], bool]=predicate, array: List[_A_]=array) -> _A_:
        while True:
            (i,) = (i_mut,)
            if i < 0:
                return index_not_found()
            
            elif predicate(array[i]):
                return array[i]
            
            else: 
                i_mut = i - 1
                continue
            
            break
    
    return loop(len(array) - 1)


def try_find_back(predicate: Callable[[_A_], bool], array: List[_A_]) -> Optional[_A_]:
    def loop(i_mut: int, predicate: Callable[[_A_], bool]=predicate, array: List[_A_]=array) -> Optional[_A_]:
        while True:
            (i,) = (i_mut,)
            if i < 0:
                return None
            
            elif predicate(array[i]):
                return some(array[i])
            
            else: 
                i_mut = i - 1
                continue
            
            break
    
    return loop(len(array) - 1)


def find_last_index(predicate: Callable[[_A_], bool], array: List[_A_]) -> int:
    def loop(i_mut: int, predicate: Callable[[_A_], bool]=predicate, array: List[_A_]=array) -> int:
        while True:
            (i,) = (i_mut,)
            if i < 0:
                return -1
            
            elif predicate(array[i]):
                return i
            
            else: 
                i_mut = i - 1
                continue
            
            break
    
    return loop(len(array) - 1)


def find_index_back(predicate: Callable[[_A_], bool], array: List[_A_]) -> int:
    def loop(i_mut: int, predicate: Callable[[_A_], bool]=predicate, array: List[_A_]=array) -> int:
        while True:
            (i,) = (i_mut,)
            if i < 0:
                return index_not_found()
            
            elif predicate(array[i]):
                return i
            
            else: 
                i_mut = i - 1
                continue
            
            break
    
    return loop(len(array) - 1)


def try_find_index_back(predicate: Callable[[_A_], bool], array: List[_A_]) -> Optional[int]:
    def loop(i_mut: int, predicate: Callable[[_A_], bool]=predicate, array: List[_A_]=array) -> Optional[int]:
        while True:
            (i,) = (i_mut,)
            if i < 0:
                return None
            
            elif predicate(array[i]):
                return i
            
            else: 
                i_mut = i - 1
                continue
            
            break
    
    return loop(len(array) - 1)


def choose(chooser: Callable[[_T], Optional[_U]], array: List[_T], cons: Cons_1[_U]) -> List[_U]:
    res : List[_U] = []
    for i in range(0, (len(array) - 1) + 1, 1):
        match_value : Optional[_U] = chooser(array[i])
        if match_value is not None:
            y : _U = value_1(match_value)
            ignore(res.append(y))
        
    if equals_1(cons, None):
        return res
    
    else: 
        def arrow_49(x: Optional[_U]=None, chooser: Callable[[_T], Optional[_U]]=chooser, array: List[_T]=array, cons: Cons_1[_U]=cons) -> _U:
            return x
        
        return map(arrow_49, res, cons)
    


def fold_indexed(folder: Callable[[int, _STATE, _T], _STATE], state: _STATE, array: List[_T]) -> _STATE:
    def arrow_50(delegate_arg0: _STATE, delegate_arg1: _T, delegate_arg2: int, folder: Callable[[int, _STATE, _T], _STATE]=folder, state: _STATE=state, array: List[_T]=array) -> _STATE:
        return folder(delegate_arg2, delegate_arg0, delegate_arg1)
    
    return array.reduce((arrow_50, state))


def fold(folder: Callable[[_STATE, _T], _STATE], state: _STATE, array: List[_T]) -> _STATE:
    def arrow_51(acc: _STATE, x: _T, folder: Callable[[_STATE, _T], _STATE]=folder, state: _STATE=state, array: List[_T]=array) -> _STATE:
        return folder(acc, x)
    
    return functools.reduce(arrow_51, array, state)


def iterate(action: Callable[[_T], None], array: List[_T]) -> None:
    for i in range(0, (len(array) - 1) + 1, 1):
        action(array[i])


def iterate_indexed(action: Callable[[int, _T], None], array: List[_T]) -> None:
    for i in range(0, (len(array) - 1) + 1, 1):
        action(i, array[i])


def iterate2(action: Callable[[_T, _T], None], array1: List[_T], array2: List[_T]) -> None:
    if len(array1) != len(array2):
        different_lengths()
    
    for i in range(0, (len(array1) - 1) + 1, 1):
        action(array1[i], array2[i])


def iterate_indexed2(action: Callable[[int, _T, _T], None], array1: List[_T], array2: List[_T]) -> None:
    if len(array1) != len(array2):
        different_lengths()
    
    for i in range(0, (len(array1) - 1) + 1, 1):
        action(i, array1[i], array2[i])


def is_empty(array: List[_T]) -> bool:
    return len(array) == 0


def for_all(predicate: Callable[[_T], bool], array: List[_T]) -> bool:
    return all([predicate(x) for x in array])


def permute(f: Callable[[int], int], array: List[_T]) -> List[_T]:
    size : int = len(array) or 0
    res : List[_T] = array[:]
    check_flags : MutableSequence[int] = [None]*size
    def arrow_52(i: int, x: _T, f: Callable[[int], int]=f, array: List[_T]=array) -> None:
        j : int = f(i) or 0
        if True if (j < 0) else (j >= size):
            raise Exception("Not a valid permutation")
        
        res[j] = x
        check_flags[j] = 1
    
    iterate_indexed(arrow_52, array)
    def predicate(y: int, f: Callable[[int], int]=f, array: List[_T]=array) -> bool:
        return 1 == y
    
    if not (all([predicate(x) for x in check_flags])):
        raise Exception("Not a valid permutation")
    
    return res


def set_slice(target: List[_T], lower: Optional[int], upper: Optional[int], source: List[_T]) -> None:
    lower_1 : int = default_arg(lower, 0) or 0
    upper_1 : int = default_arg(upper, 0) or 0
    length : int = ((upper_1 if (upper_1 > 0) else (len(target) - 1)) - lower_1) or 0
    for i in range(0, length + 1, 1):
        target[i + lower_1] = source[i]


def sort_in_place_by(projection: Callable[[_A], _B], xs: List[_A], comparer: IComparer[_B]) -> None:
    def arrow_53(x: _A, y: _A, projection: Callable[[_A], _B]=projection, xs: List[_A]=xs, comparer: IComparer[_B]=comparer) -> int:
        return comparer.Compare(projection(x), projection(y))
    
    xs.sort()


def sort_in_place(xs: List[_T], comparer: IComparer[_T]) -> None:
    def arrow_54(x: _T, y: _T, xs: List[_T]=xs, comparer: IComparer[_T]=comparer) -> int:
        return comparer.Compare(x, y)
    
    xs.sort()


def sort(xs: List[_T], comparer: IComparer[_T]) -> List[_T]:
    xs_1 : List[_T] = xs[:]
    def comparer_1(x: _T, y: _T, xs: List[_T]=xs, comparer: IComparer[_T]=comparer) -> int:
        return comparer.Compare(x, y)
    
    xs_1.sort()
    return xs_1


def sort_by(projection: Callable[[_A], _B], xs: List[_A], comparer: IComparer[_B]) -> List[_A]:
    xs_1 : List[_A] = xs[:]
    def expr_55():
        def comparer_1(x: _A, y: _A, projection: Callable[[_A], _B]=projection, xs: List[_A]=xs, comparer: IComparer[_B]=comparer) -> int:
            return comparer.Compare(projection(x), projection(y))
        
        xs_1.sort()
        return xs_1
    
    return expr_55()


def sort_descending(xs: List[_T], comparer: IComparer[_T]) -> List[_T]:
    xs_1 : List[_T] = xs[:]
    def comparer_1(x: _T, y: _T, xs: List[_T]=xs, comparer: IComparer[_T]=comparer) -> int:
        return comparer.Compare(x, y) * -1
    
    xs_1.sort()
    return xs_1


def sort_by_descending(projection: Callable[[_A], _B], xs: List[_A], comparer: IComparer[_B]) -> List[_A]:
    xs_1 : List[_A] = xs[:]
    def expr_56():
        def comparer_1(x: _A, y: _A, projection: Callable[[_A], _B]=projection, xs: List[_A]=xs, comparer: IComparer[_B]=comparer) -> int:
            return comparer.Compare(projection(x), projection(y)) * -1
        
        xs_1.sort()
        return xs_1
    
    return expr_56()


def sort_with(comparer: Callable[[_T, _T], int], xs: List[_T]) -> List[_T]:
    comparer_1 : Callable[[_T, _T], int] = comparer
    xs_1 : List[_T] = xs[:]
    xs_1.sort()
    return xs_1


def all_pairs(xs: List[_T1], ys: List[_T2]) -> List[Tuple[_T1, _T2]]:
    len1 : int = len(xs) or 0
    len2 : int = len(ys) or 0
    res : List[Tuple[_T1, _T2]] = [None]*(len1 * len2)
    for i in range(0, (len(xs) - 1) + 1, 1):
        for j in range(0, (len(ys) - 1) + 1, 1):
            res[(i * len2) + j] = (xs[i], ys[j])
    return res


def unfold(generator: Callable[[_STATE], Optional[Tuple[_T, _STATE]]], state: _STATE) -> List[_T]:
    res : List[_T] = []
    def loop(state_1_mut: Optional[_STATE]=None, generator: Callable[[_STATE], Optional[Tuple[_T, _STATE]]]=generator, state: _STATE=state) -> None:
        while True:
            (state_1,) = (state_1_mut,)
            match_value : Optional[Tuple[_T, _STATE]] = generator(state_1)
            if match_value is not None:
                x : _T = match_value[0]
                s : _STATE = match_value[1]
                ignore(res.append(x))
                state_1_mut = s
                continue
            
            break
    
    loop(state)
    return res


def unzip(array: List[Tuple[_A_, _B_]]) -> Tuple[List[_A_], List[_B_]]:
    len_1 : int = len(array) or 0
    res1 : List[_A_] = [None]*len_1
    res2 : List[_B_] = [None]*len_1
    def arrow_57(i: int, tupled_arg: Tuple[_A_, _B_], array: List[Tuple[_A_, _B_]]=array) -> None:
        res1[i] = tupled_arg[0]
        res2[i] = tupled_arg[1]
    
    iterate_indexed(arrow_57, array)
    return (res1, res2)


def unzip3(array: List[Tuple[_A_, _B_, _C_]]) -> Tuple[List[_A_], List[_B_], List[_C_]]:
    len_1 : int = len(array) or 0
    res1 : List[_A_] = [None]*len_1
    res2 : List[_B_] = [None]*len_1
    res3 : List[_C_] = [None]*len_1
    def arrow_58(i: int, tupled_arg: Tuple[_A_, _B_, _C_], array: List[Tuple[_A_, _B_, _C_]]=array) -> None:
        res1[i] = tupled_arg[0]
        res2[i] = tupled_arg[1]
        res3[i] = tupled_arg[2]
    
    iterate_indexed(arrow_58, array)
    return (res1, res2, res3)


def zip(array1: List[_T], array2: List[_U]) -> List[Tuple[_T, _U]]:
    if len(array1) != len(array2):
        different_lengths()
    
    result : List[Tuple[_T, _U]] = [None]*len(array1)
    for i in range(0, (len(array1) - 1) + 1, 1):
        result[i] = (array1[i], array2[i])
    return result


def zip3(array1: List[_T], array2: List[_U], array3: List[_U]) -> List[Tuple[_T, _U, _U]]:
    if True if (len(array1) != len(array2)) else (len(array2) != len(array3)):
        different_lengths()
    
    result : List[Tuple[_T, _U, _U]] = [None]*len(array1)
    for i in range(0, (len(array1) - 1) + 1, 1):
        result[i] = (array1[i], array2[i], array3[i])
    return result


def chunk_by_size(chunk_size: int, array: List[_T]) -> List[List[_T]]:
    if chunk_size < 1:
        raise Exception("The input must be positive.\\nParameter name: size")
    
    if len(array) == 0:
        return [[]]
    
    else: 
        result : List[List[_T]] = []
        for x in range(0, (int(math.ceil(len(array) / chunk_size)) - 1) + 1, 1):
            start : int = (x * chunk_size) or 0
            slice : List[_T] = array[start:start+chunk_size]
            ignore(result.append(slice))
        return result
    


def split_at(index: int, array: List[_T]) -> Tuple[List[_T], List[_T]]:
    if True if (index < 0) else (index > len(array)):
        raise Exception((SR_indexOutOfBounds + "\\nParameter name: ") + "index")
    
    return (array[0:0+index], array[index:])


def compare_with(comparer: Callable[[_T, _T], int], array1: List[_T], array2: List[_T]) -> int:
    if array1 is None:
        if array2 is None:
            return 0
        
        else: 
            return -1
        
    
    elif array2 is None:
        return 1
    
    else: 
        i : int = 0
        result : int = 0
        length1 : int = len(array1) or 0
        length2 : int = len(array2) or 0
        if length1 > length2:
            return 1
        
        elif length1 < length2:
            return -1
        
        else: 
            while (result == 0) if (i < length1) else False:
                result = comparer(array1[i], array2[i]) or 0
                i = (i + 1) or 0
            return result
        
    


def equals_with(equals: Callable[[_T, _T], bool], array1: List[_T], array2: List[_T]) -> bool:
    if array1 is None:
        if array2 is None:
            return True
        
        else: 
            return False
        
    
    elif array2 is None:
        return False
    
    else: 
        i : int = 0
        result : bool = True
        length1 : int = len(array1) or 0
        length2 : int = len(array2) or 0
        if length1 > length2:
            return False
        
        elif length1 < length2:
            return False
        
        else: 
            while result if (i < length1) else False:
                result = equals(array1[i], array2[i])
                i = (i + 1) or 0
            return result
        
    


def exactly_one(array: List[_T]) -> _T:
    if len(array) == 1:
        return array[0]
    
    elif len(array) == 0:
        raise Exception("The input sequence was empty\\nParameter name: array")
    
    else: 
        raise Exception("Input array too long\\nParameter name: array")
    


def try_exactly_one(array: List[_T]) -> Optional[_T]:
    if len(array) == 1:
        return some(array[0])
    
    else: 
        return None
    


def head(array: List[_T]) -> _T:
    if len(array) == 0:
        raise Exception("The input array was empty\\nParameter name: array")
    
    else: 
        return array[0]
    


def try_head(array: List[_T]) -> Optional[_T]:
    if len(array) == 0:
        return None
    
    else: 
        return some(array[0])
    


def tail(array: List[_T]) -> List[_T]:
    if len(array) == 0:
        raise Exception("Not enough elements\\nParameter name: array")
    
    return array[1:]


def item(index: int, array: List[_A_]) -> _A_:
    return array[index]


def try_item(index: int, array: List[_T]) -> Optional[_T]:
    if True if (index < 0) else (index >= len(array)):
        return None
    
    else: 
        return some(array[index])
    


def fold_back_indexed(folder: Callable[[int, _T, _STATE], _STATE], array: List[_T], state: _STATE) -> _STATE:
    def arrow_59(delegate_arg0: _STATE, delegate_arg1: _T, delegate_arg2: int, folder: Callable[[int, _T, _STATE], _STATE]=folder, array: List[_T]=array, state: _STATE=state) -> _STATE:
        return folder(delegate_arg2, delegate_arg1, delegate_arg0)
    
    return array.reduceRight((arrow_59, state))


def fold_back(folder: Callable[[_T, _STATE], _STATE], array: List[_T], state: _STATE) -> _STATE:
    def arrow_60(acc: _STATE, x: _T, folder: Callable[[_T, _STATE], _STATE]=folder, array: List[_T]=array, state: _STATE=state) -> _STATE:
        return folder(x, acc)
    
    return functools.reduce(arrow_60, array[::-1], state)


def fold_indexed2(folder: Callable[[int, _A_, _B_, _C_], _A_], state: _A_, array1: List[_B_], array2: List[_C_]) -> _A_:
    acc : _A_ = state
    if len(array1) != len(array2):
        raise Exception("Arrays have different lengths")
    
    for i in range(0, (len(array1) - 1) + 1, 1):
        acc = folder(i, acc, array1[i], array2[i])
    return acc


def fold2(folder: Callable[[_STATE, _T1, _T2], _STATE], state: _STATE, array1: List[_T1], array2: List[_T2]) -> _STATE:
    def arrow_61(_arg1: int, acc: _STATE, x: _T1, y: _T2, folder: Callable[[_STATE, _T1, _T2], _STATE]=folder, state: _STATE=state, array1: List[_T1]=array1, array2: List[_T2]=array2) -> _STATE:
        return folder(acc, x, y)
    
    return fold_indexed2(arrow_61, state, array1, array2)


def fold_back_indexed2(folder: Callable[[int, _T1, _T2, _STATE], _STATE], array1: List[_T1], array2: List[_T2], state: _STATE) -> _STATE:
    acc : _STATE = state
    if len(array1) != len(array2):
        different_lengths()
    
    size : int = len(array1) or 0
    for i in range(1, size + 1, 1):
        acc = folder(i - 1, array1[size - i], array2[size - i], acc)
    return acc


def fold_back2(f: Callable[[_T1, _T2, _STATE], _STATE], array1: List[_T1], array2: List[_T2], state: _STATE) -> _STATE:
    def arrow_62(_arg1: int, x: _T1, y: _T2, acc: _STATE, f: Callable[[_T1, _T2, _STATE], _STATE]=f, array1: List[_T1]=array1, array2: List[_T2]=array2, state: _STATE=state) -> _STATE:
        return f(x, y, acc)
    
    return fold_back_indexed2(arrow_62, array1, array2, state)


def reduce(reduction: Callable[[_T, _T], _T], array: List[_T]) -> _T:
    if len(array) == 0:
        raise Exception("The input array was empty")
    
    return functools.reduce(reduction, array)


def reduce_back(reduction: Callable[[_T, _T], _T], array: List[_T]) -> _T:
    if len(array) == 0:
        raise Exception("The input array was empty")
    
    return functools.reduce(reduction, array[::-1])


def for_all2(predicate: Callable[[_A_, _B_], bool], array1: List[_A_], array2: List[_B_]) -> bool:
    def arrow_63(acc: bool, x: _A_, y: _B_, predicate: Callable[[_A_, _B_], bool]=predicate, array1: List[_A_]=array1, array2: List[_B_]=array2) -> bool:
        return predicate(x, y) if acc else False
    
    return fold2(arrow_63, True, array1, array2)


def exists_offset(predicate_mut: Callable[[_T], bool], array_mut: List[_T], index_mut: int) -> bool:
    while True:
        (predicate, array, index) = (predicate_mut, array_mut, index_mut)
        if index == len(array):
            return False
        
        elif predicate(array[index]):
            return True
        
        else: 
            predicate_mut = predicate
            array_mut = array
            index_mut = index + 1
            continue
        
        break


def exists(predicate: Callable[[_A_], bool], array: List[_A_]) -> bool:
    return exists_offset(predicate, array, 0)


def exists_offset2(predicate_mut: Callable[[_A_, _B_], bool], array1_mut: List[_A_], array2_mut: List[_B_], index_mut: int) -> bool:
    while True:
        (predicate, array1, array2, index) = (predicate_mut, array1_mut, array2_mut, index_mut)
        if index == len(array1):
            return False
        
        elif predicate(array1[index], array2[index]):
            return True
        
        else: 
            predicate_mut = predicate
            array1_mut = array1
            array2_mut = array2
            index_mut = index + 1
            continue
        
        break


def exists2(predicate: Callable[[_A_, _B_], bool], array1: List[_A_], array2: List[_B_]) -> bool:
    if len(array1) != len(array2):
        different_lengths()
    
    return exists_offset2(predicate, array1, array2, 0)


def sum(array: List[_T], adder: IGenericAdder_1[_T]) -> _T:
    acc : _T = adder.GetZero()
    for i in range(0, (len(array) - 1) + 1, 1):
        acc = adder.Add(acc, array[i])
    return acc


def sum_by(projection: Callable[[_T], _T2], array: List[_T], adder: IGenericAdder_1[_T2]) -> _T2:
    acc : _T2 = adder.GetZero()
    for i in range(0, (len(array) - 1) + 1, 1):
        acc = adder.Add(acc, projection(array[i]))
    return acc


def max_by(projection: Callable[[_A], _B], xs: List[_A], comparer: IComparer[_B]) -> _A:
    def arrow_64(x: _A, y: _A, projection: Callable[[_A], _B]=projection, xs: List[_A]=xs, comparer: IComparer[_B]=comparer) -> _A:
        return y if (comparer.Compare(projection(y), projection(x)) > 0) else x
    
    return reduce(arrow_64, xs)


def max(xs: List[_A], comparer: IComparer[_A]) -> _A:
    def arrow_65(x: _A, y: _A, xs: List[_A]=xs, comparer: IComparer[_A]=comparer) -> _A:
        return y if (comparer.Compare(y, x) > 0) else x
    
    return reduce(arrow_65, xs)


def min_by(projection: Callable[[_A], _B], xs: List[_A], comparer: IComparer[_B]) -> _A:
    def arrow_66(x: _A, y: _A, projection: Callable[[_A], _B]=projection, xs: List[_A]=xs, comparer: IComparer[_B]=comparer) -> _A:
        return x if (comparer.Compare(projection(y), projection(x)) > 0) else y
    
    return reduce(arrow_66, xs)


def min(xs: List[_A], comparer: IComparer[_A]) -> _A:
    def arrow_67(x: _A, y: _A, xs: List[_A]=xs, comparer: IComparer[_A]=comparer) -> _A:
        return x if (comparer.Compare(y, x) > 0) else y
    
    return reduce(arrow_67, xs)


def average(array: List[_T], averager: IGenericAverager_1[_T]) -> _T:
    if len(array) == 0:
        raise Exception("The input array was empty\\nParameter name: array")
    
    total : _T = averager.GetZero()
    for i in range(0, (len(array) - 1) + 1, 1):
        total = averager.Add(total, array[i])
    return averager.DivideByInt(total, len(array))


def average_by(projection: Callable[[_T], _T2], array: List[_T], averager: IGenericAverager_1[_T2]) -> _T2:
    if len(array) == 0:
        raise Exception("The input array was empty\\nParameter name: array")
    
    total : _T2 = averager.GetZero()
    for i in range(0, (len(array) - 1) + 1, 1):
        total = averager.Add(total, projection(array[i]))
    return averager.DivideByInt(total, len(array))


def windowed(window_size: int, source: List[_T]) -> List[List[_T]]:
    if window_size <= 0:
        raise Exception("windowSize must be positive")
    
    res : List[List[_T]]
    def arrow_68(x: int, y: int, window_size: int=window_size, source: List[_T]=source) -> int:
        return compare_primitives(x, y)
    
    len_1 : int = max_1(arrow_68, 0, len(source) - window_size) or 0
    res = [None]*len_1
    for i in range(window_size, len(source) + 1, 1):
        res[i - window_size] = source[i - window_size:(i - 1) + 1]
    return res


def split_into(chunks: int, array: List[_T]) -> List[List[_T]]:
    if chunks < 1:
        raise Exception("The input must be positive.\\nParameter name: chunks")
    
    if len(array) == 0:
        return [[]]
    
    else: 
        result : List[List[_T]] = []
        def arrow_69(x: int, y: int, chunks: int=chunks, array: List[_T]=array) -> int:
            return compare_primitives(x, y)
        
        chunks_1 : int = min_1(arrow_69, chunks, len(array)) or 0
        min_chunk_size : int = (len(array) // chunks_1) or 0
        chunks_with_extra_item : int = (len(array) % chunks_1) or 0
        for i in range(0, (chunks_1 - 1) + 1, 1):
            chunk_size : int = ((min_chunk_size + 1) if (i < chunks_with_extra_item) else min_chunk_size) or 0
            def arrow_70(x_1: int, y_1: int, chunks: int=chunks, array: List[_T]=array) -> int:
                return compare_primitives(x_1, y_1)
            
            start : int = ((i * min_chunk_size) + min_1(arrow_70, chunks_with_extra_item, i)) or 0
            slice : List[_T] = array[start:start+chunk_size]
            ignore(result.append(slice))
        return result
    


def transpose(arrays: IEnumerable[List[_T]], cons: Cons_1[_T]) -> List[List[_T]]:
    arrays_1 : List[List[_T]] = arrays if (isinstance(arrays, list)) else (list(arrays))
    len_1 : int = len(arrays_1) or 0
    if len_1 == 0:
        return [None]*0
    
    else: 
        len_inner : int = len(arrays_1[0]) or 0
        def predicate(a: List[_T], arrays: IEnumerable[List[_T]]=arrays, cons: Cons_1[_T]=cons) -> bool:
            return len(a) == len_inner
        
        if not for_all(predicate, arrays_1):
            different_lengths()
        
        result : List[List[_T]] = [None]*len_inner
        for i in range(0, (len_inner - 1) + 1, 1):
            result[i] = Helpers_allocateArrayFromCons(cons, len_1)
            for j in range(0, (len_1 - 1) + 1, 1):
                result[i][j] = arrays_1[j][i]
        return result
    


def insert_at(index: int, y: _T, xs: List[_T]) -> List[_T]:
    len_1 : int = len(xs) or 0
    if True if (index < 0) else (index > len_1):
        raise Exception((SR_indexOutOfBounds + "\\nParameter name: ") + "index")
    
    target : List[_T] = [x for i, x in enumerate(xs) if i < (len_1 + 1)]
    for i in range(0, (index - 1) + 1, 1):
        target[i] = xs[i]
    target[index] = y
    for i_1 in range(index, (len_1 - 1) + 1, 1):
        target[i_1 + 1] = xs[i_1]
    return target


def insert_many_at(index: int, ys: IEnumerable[_T], xs: List[_T]) -> List[_T]:
    len_1 : int = len(xs) or 0
    if True if (index < 0) else (index > len_1):
        raise Exception((SR_indexOutOfBounds + "\\nParameter name: ") + "index")
    
    ys_1 : List[_T] = list(ys)
    len2 : int = len(ys_1) or 0
    target : List[_T] = [x for i, x in enumerate(xs) if i < (len_1 + len2)]
    for i in range(0, (index - 1) + 1, 1):
        target[i] = xs[i]
    for i_1 in range(0, (len2 - 1) + 1, 1):
        target[index + i_1] = ys_1[i_1]
    for i_2 in range(index, (len_1 - 1) + 1, 1):
        target[i_2 + len2] = xs[i_2]
    return target


def remove_at(index: int, xs: List[_T]) -> List[_T]:
    if True if (index < 0) else (index >= len(xs)):
        raise Exception((SR_indexOutOfBounds + "\\nParameter name: ") + "index")
    
    i : int = -1
    def predicate(_arg1: Optional[_T]=None, index: int=index, xs: List[_T]=xs) -> bool:
        nonlocal i
        i = (i + 1) or 0
        return i != index
    
    return filter(predicate, xs)


def remove_many_at(index: int, count: int, xs: List[_T]) -> List[_T]:
    i : int = -1
    status : int = -1
    def predicate(_arg1: Optional[_T]=None, index: int=index, count: int=count, xs: List[_T]=xs) -> bool:
        nonlocal i, status
        i = (i + 1) or 0
        if i == index:
            status = 0
            return False
        
        elif i > index:
            if i < (index + count):
                return False
            
            else: 
                status = 1
                return True
            
        
        else: 
            return True
        
    
    ys : List[_T] = filter(predicate, xs)
    status_1 : int = (1 if (((i + 1) == (index + count)) if (status == 0) else False) else status) or 0
    if status_1 < 1:
        raise Exception((SR_indexOutOfBounds + "\\nParameter name: ") + ("index" if (status_1 < 0) else "count"))
    
    return ys


def update_at(index: int, y: _T, xs: List[_T]) -> List[_T]:
    len_1 : int = len(xs) or 0
    if True if (index < 0) else (index >= len_1):
        raise Exception((SR_indexOutOfBounds + "\\nParameter name: ") + "index")
    
    target : List[_T] = [x for i, x in enumerate(xs) if i < len_1]
    for i in range(0, (len_1 - 1) + 1, 1):
        target[i] = y if (i == index) else xs[i]
    return target


